-------------------------------------------------------------------------------
-- FileName: DBMS_XMLGEN_1.sql
-- Description: 
--   Samples demonstrate the DBMS_XMLGEN functionality
-- Modified (MM/DD/YYYY)
--   jiwang  04/25/2003 - Add Binding Variable Sample
--   jiwang  04/20/2003 - Create
-- Notes:
--  - Login into HR schema to run the samples
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Example 1: Simple Query
-------------------------------------------------------------------------------
SET autoprint ON
SET LONG 100000
VAR result CLOB
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
BEGIN
   qryCtx := 
     DBMS_XMLGEN.newContext('select employee_id,
                                    first_name, 
                                    last_name, 
                                    email,
                                    phone_number
                             from employees 
                             where salary > 20000');
   :result := DBMS_XMLGEN.getXML(qryCtx);
   DBMS_XMLGEN.CloseContext(qryCtx);
END;                 
/

-------------------------------------------------------------------------------
-- Example 2: Format Setting
-------------------------------------------------------------------------------
SET autoprint ON
SET LONG 100000
VAR result CLOB
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
BEGIN
   qryCtx := 
     DBMS_XMLGEN.newContext('select employee_id as "@id",
                                    first_name as "FirstName", 
                                    last_name as "LastName", 
                                    email as "Email",
                                    phone_number as "Phone"
                             from employees 
                             where salary > 20000');
   DBMS_XMLGEN.setRowSetTag(qryCtx,'EMPLOYEES');
   DBMS_XMLGEN.setRowTag(qryCtx,'EMPLOYEE');
   :result := DBMS_XMLGEN.getXML(qryCtx);
   DBMS_XMLGEN.CloseContext(qryCtx);
END;                 
/

-------------------------------------------------------------------------------
-- Example 3: Print Format
-------------------------------------------------------------------------------
SET autoprint ON
SET LONG 100000
VAR result CLOB
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
BEGIN
   qryCtx := 
     DBMS_XMLGEN.newContext('select *
                             from employees 
                             where salary > 20000');
   DBMS_XMLGEN.setPrettyPrinting(qryCtx, FALSE);
   :result := DBMS_XMLGEN.getXML(qryCtx);
   DBMS_XMLGEN.CloseContext(qryCtx);
END;                 
/
-------------------------------------------------------------------------------
-- Example 4: Fetching Data
-------------------------------------------------------------------------------
CREATE TABLE temp_clob_tab(result CLOB);
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
   result CLOB;
BEGIN
   -- Open Context
   qryCtx := DBMS_XMLGEN.newContext('select * from employees order by employee_id');
   DBMS_XMLGEN.setMaxRows(qryCtx,10);
   LOOP
      result:= DBMS_XMLGEN.getXML(qryCtx);
      EXIT WHEN DBMS_XMLGEN.getNumRowsProcessed(qryCtx) =0;
      INSERT INTO temp_clob_tab VALUES(result);
   END LOOP;
   -- Close Context
   DBMS_XMLGEN.closeContext(qryCtx);
END;
/

-------------------------------------------------------------------------------
-- Example 5: newcontextFromHierarchy
-------------------------------------------------------------------------------
SET autoprint ON
SET LONG 100000
VAR result CLOB
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
BEGIN
   qryCtx := 
     DBMS_XMLGEN.newcontextFromHierarchy('SELECT level, XMLELEMENT(emp, 
                                    XMLATTRIBUTES(employee_id),
                                    XMLFOREST(first_name, last_name)) as xml
                                  FROM employees connect by prior
                                  employee_id = manager_id start with employee_id=100');
   :result := DBMS_XMLGEN.getXML(qryCtx);
   DBMS_XMLGEN.CloseContext(qryCtx);
END;
/



