-------------------------------------------------------------------------------
-- FileName: DBMS_XMLGEN_2.sql
-- Description: 
--   Samples demonstrate the DBMS_XMLGEN functionality
-- Modified (MM/DD/YYYY)
--   jiwang  04/25/2003 - Add Binding Variable Sample
--   jiwang  04/20/2003 - Create
-- Notes:
--  - Login into HR schema to run the samples
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Example 6: Binding Variables
-------------------------------------------------------------------------------
drop table temp;
create table temp(doc clob);
DECLARE
   qryCtx DBMS_XMLGEN.ctxType;
   v_sql VARCHAR2(100);
   v_clob clob;
   TYPE deptType IS RECORD (department_id NUMBER(4)); 
   v_deprec deptType;
   v_refcur SYS_REFCURSOR;
BEGIN
  qryCtx := DBMS_XMLGEN.newContext('SELECT * FROM employees WHERE department_id = :MID');

   -- Open Ref Cursor
  OPEN v_refcur FOR 'SELECT department_id FROM departments WHERE location_id = 1700';
  -- Loop each department and get
  LOOP
    -- Fetch from cursor variable.
   FETCH v_refcur INTO v_deprec;
   EXIT WHEN v_refcur%NOTFOUND; 
   DBMS_XMLGEN.setBindValue( qryCtx, 'MID', v_deprec.department_id);
   v_clob := DBMS_XMLGEN.getXML( qryCtx );
   INSERT INTO temp VALUES(v_clob);
  END LOOP;
  DBMS_XMLGEN.CLOSEContext(qryCtx);
EXCEPTION
   WHEN OTHERS THEN
     DBMS_XMLGEN.closeContext(qryCtx);
END;
/
show errors;

SELECT parse_calls,executions
   FROM v$sql
   WHERE sql_text LIKE 'SELECT * FROM employees WHERE department_id =%';

-------------------------------------------------------------------------------
-- Example 7: Create Resources Files
-------------------------------------------------------------------------------
SET serveroutput ON SIZE 1000000
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
   res BOOLEAN;
   result XMLType;
   i NUMBER :=0; 
BEGIN
  -- Create the query context
  qryCtx := DBMS_XMLGEN.newContext('select * from employees order by employee_id');
  DBMS_XMLGEN.setMaxRows(qryCtx,10);
  LOOP
   result:= DBMS_XMLGEN.getXMLType(qryCtx);
   EXIT WHEN DBMS_XMLGEN.getNumRowsProcessed(qryCtx) =0;
   i :=i+1;
   -- Create XML DB Resources
   res := DBMS_XDB.CreateResource('/public/emp'||i||'.xml',result);
   IF res = FALSE THEN
      DBMS_OUTPUT.put_line('Error creating XML DB resource');
   ELSE
      DBMS_OUTPUT.put_line('/public/emp'||i||'.xml'||' is created');
   END IF;
  END LOOP;
  DBMS_XMLGEN.CloseContext(qryCtx);
END;
/

-------------------------------------------------------------------------------
-- Example 8: Merge the tow query result using SYS_REFCURSOR
-------------------------------------------------------------------------------
SET autoprint ON
var result CLOB;
DECLARE
  salary NUMBER :=20000;
  refcur SYS_REFCURSOR;
  qryCtx DBMS_XMLGEN.ctxHandle;
BEGIN
   OPEN refcur FOR 'select * from employees where salary> :1' using salary;
   qryCtx := DBMS_XMLGEN.newContext(refcur);
   :result := DBMS_XMLGEN.getXML(qryCtx,DBMS_XMLGEN.NONE);
   DBMS_XMLGEN.CloseContext(qryCtx);
END;
/

-------------------------------------------------------------------------------
-- Example 9: Special Characters
-------------------------------------------------------------------------------
SELECT DBMS_XMLGEN.getXML('SELECT ''Don''''t escape me!'' as result FROM dual') 
FROM dual;

SET autoprint ON
SET LONG 100000
VAR result CLOB
DECLARE
   qryCtx DBMS_XMLGEN.ctxHandle;
BEGIN
   qryCtx := 
     DBMS_XMLGEN.newContext('SELECT ''Don''''t escape me!'' as result FROM dual');
   DBMS_XMLGEN.SetConvertSpecialChars(qryCtx, FALSE);
   :result := DBMS_XMLGEN.getXML(qryCtx);
   DBMS_XMLGEN.CloseContext(qryCtx);
END;                 
/
show errors;

