-------------------------------------------------------------------------------
-- FileName: GenerateSchema.sql
-- Description: 
--   Generate XML Schema from Oracle Database
-- Modified (MM/DD/YYYY)
--   jiwang  06/25/2003 - Create
-------------------------------------------------------------------------------
set echo on
set long 100000
-------------------------------------------------------------------------------
--  Example 1: Generate XML Schema using DBMS_XMLQUERY
-------------------------------------------------------------------------------
set autoprint on
var xmldoc clob
begin
 :xmldoc := dbms_xmlquery.getXML(
          'select employee_id as "@id", 
                  first_name, last_name 
           from employees
           where rownum=0', 
          DBMS_XMLQUERY.SCHEMA);
end;
/
pause;
-------------------------------------------------------------------------------
--  Example 2: Generate XML Schema using DBMS_XMLGEN (Not Supported)
-------------------------------------------------------------------------------
set autoprint on
var result clob;
begin
 :result:=DBMS_XMLGEN.getXML('select * from employees where rownum=0',DBMS_XMLGEN.SCHEMA);
end;
/

pause;
-------------------------------------------------------------------------------
--  Example 3: Generate DTD uisng DBMS_XMLQUERY
-------------------------------------------------------------------------------
set autoprint on
var result clob;
begin
 :result:=DBMS_XMLQUERY.getXML('select employee_id as "@id", email, 
                                department_id 
                                from employees where rownum=1',
                                DBMS_XMLQUERY.DTD);
end;
/
pause;
-------------------------------------------------------------------------------
--  Example 4: Generate XML Schema with query against Multiple Tables
-------------------------------------------------------------------------------
set autoprint on
var result clob;
begin
 :result:=DBMS_XMLQUERY.getXML('select employee_id as "@id", email, 
                                a.department_id from employees, departments a 
                                where rownum=1',
                                DBMS_XMLQUERY.SCHEMA);
end;
/
pause;
-------------------------------------------------------------------------------
--  Example 5: Gnerate XML Schema using DBMS_XMLSchema (Not Supported)
-------------------------------------------------------------------------------
set autoprint on
var result clob;
begin
 :result:=DBMS_XMLSCHEMA.generateSchema('HR',
                                  'select * from employees,sh.sales where rownum<1');
end;
/
pause;
-------------------------------------------------------------------------------
--  Example 6: Generate XML Schema using DBMS_XMLSCHEMA
-------------------------------------------------------------------------------
create or replace type emp_t as object
(
  "@id"  number(6),
  email  varchar2(25),
  department_id  number(4));
/
SELECT DBMS_XMLSchema.generateSchema('HR', 'EMP_T') as result from dual;

pause;
-------------------------------------------------------------------------------
--  Example 7: Generate XML Schema from Complex SQL Query (??)
-------------------------------------------------------------------------------
create or replace type v_emp_t as varray(3) of emp_t
/
set autoprint on
var result clob;
begin
 :result:=DBMS_XMLQUERY.getXML('SELECT CAST(
                         MULTISET(SELECT employee_id as "@id",email,department_id 
                                  FROM employees 
                                  where rownum<3) AS v_emp_t) as result 
                        FROM dual', DBMS_XMLQUERY.SCHEMA);
end;
/
pause;
-------------------------------------------------------------------------------
--  Example 8: Generate a List of XML Schemas
-------------------------------------------------------------------------------
create or replace type address_t as object
(
  street varchar2(25),
  city varchar2(10),
  state varchar2(20),
  zipcode varchar2(15));
/  

create or replace type emp_t as object
 (employee_id  number(6), 
  email  varchar2(25), 
  department_id  number(4),
  address address_t);
/

set autoprint on 
var result clob
declare
    schema   varchar2(30) := 'HR';
    typename varchar2(30) := 'ADDRESS_T';
    typevar  xmltype;
    typearr  sys.xmlsequencetype;
begin

    typearr := dbms_xmlschema.generateschemas(
                 schemaName=>schema,
                 typeName=>typename,
                 elementName=>typename,
                 schemaURL=>'http://www.example.com');

    dbms_output.put_line('There are '||typearr.count||' element(s) in typearr');

    typevar := typearr(1);
    select typevar.getClobVal() into :result from dual;
end;
/
