-------------------------------------------------------------------------------
-- FileName: updateXMLDBMIME.sql
-- Description: 
--   Update MIME by adding SVG support in XML DB
-- Modified (MM/DD/YYYY)
--   jiwang  06/11/2004 - Create
-------------------------------------------------------------------------------
set echo on 
set long 100000

CREATE OR REPLACE PROCEDURE AddNewElement(p_xpath IN VARCHAR2,
                                        p_content IN VARCHAR2,
                                        p_namespace IN VARCHAR2) AS
  v_config XMLType;
  v_mimemap XMLType;
  v_doc DBMS_XMLDOM.DOMDocument; 
  v_doc_elem  dbms_xmldom.DOMElement; 
  v_xsl  DBMS_XSLPROCESSOR.Processor; 
  v_subdoc dbms_xmldom.DOMDocument; 
  v_subdoc_elem dbms_xmldom.DOMElement; 
  v_node dbms_xmldom.DOMNode; 
  v_impnode dbms_xmldom.DOMNode; 
BEGIN
  v_mimemap := XMLType.createXML(p_content); 
  v_config := DBMS_XDB.CFG_GET();
  v_doc := DBMS_XMLDOM.newDOMDocument(v_config); 
  v_doc_elem := DBMS_XMLDOM.getDocumentElement(v_doc); 
  v_node := DBMS_XSLPROCESSOR.selectSingleNode(DBMS_XMLDOM.makeNode(v_doc),
           p_xpath, p_namespace); 
  v_subdoc :=DBMS_XMLDOM.newDOMDocument(v_mimemap); 
  v_subdoc_elem := DBMS_XMLDOM.getDocumentElement(v_subdoc); 
  v_impnode := DBMS_XMLDOM.importNode(v_doc, 
    DBMS_XMLDOM.makeNode(v_subdoc_elem), true);
  v_node := DBMS_XMLDOM.appendChild(v_node, v_impnode); 
  DBMS_XDB.CFG_UPDATE(v_config);
END;
/
show errors;

-- Check the current MIME support
SELECT distinct
  ExtractValue(value(a), '/mime-type/text()', 'xmlns="http://xmlns.oracle.com/xdb/xdbconfig.xsd"')
   AS MIMEType
FROM table(select XMLSEQUENCE(EXTRACT(DBMS_XDB.CFG_GET(),
   '/xdbconfig/sysconfig/protocolconfig/common/extension-mappings/mime-mappings/mime-mapping/mime-type')) from dual) a;

-- Add a new MIME mapping for SVG files:
BEGIN
  AddNewElement(
  '/xdbconfig/sysconfig/protocolconfig/common/extension-mappings/mime-mappings', 
  '<mime-mapping xmlns="http://xmlns.oracle.com/xdb/xdbconfig.xsd"> <extension>svg</extension>
   <mime-type>image/svg+xml</mime-type></mime-mapping>',
   'xmlns="http://xmlns.oracle.com/xdb/xdbconfig.xsd"');
END;
/
-- Check the current MIME support
SELECT distinct
  ExtractValue(value(a), '/mime-type/text()', 'xmlns="http://xmlns.oracle.com/xdb/xdbconfig.xsd"')
   AS MIMEType
FROM table(select XMLSEQUENCE(EXTRACT(DBMS_XDB.CFG_GET(),
   '/xdbconfig/sysconfig/protocolconfig/common/extension-mappings/mime-mappings/mime-mapping/mime-type')) from dual) a;


-- End of checkXMLDBMIME.sql
set echo off