<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:ns="http://www.oracle.com/faq.xsd">

  <xsl:include href="UtilText.xsl" />

  <xsl:output type="html" indent="yes"/>

  <xsl:template match="/">
   <html>
     <head>
       <meta name="keywords" 
          content="XML, XSLT, DOM, Schema, PL/SQL, XSQL Servlet, Java Bean, XMLType, Oracle Text"/>
       <meta name="description" content="FAQ document for Oracle XML Developer's Kits"/>
       <title>Oracle9i XML--FAQ</title>
       <link rel="stylesheet" href="/xdkus/js/xdkus.css"/>
      </head>
      <body>
       <table width="100%">
        <tr>
           <td colspan="2"><a  NAME="top"/><h1>Oracle XML FAQs</h1>
           </td>
        </tr>
        <tr>
          <td colspan="2">
            <xsl:apply-templates/>
          </td>
        </tr>
        <tr>
          <td colspan="2"><hr size="1"/></td>
        </tr>
        <tr>
          <td class="OraCopyright">Copyright &#169; 2002 by Oracle Corporation. All rights reserved.</td>
        </tr>
       </table>
      </body>
     </html>
   </xsl:template>
 
   <xsl:template match="section[not(@id='content')]">
     <p><h2>
        <xsl:number level="any" count="section" format="1."/>
           <xsl:text>0 </xsl:text>
        <a name="{@id}"/><xsl:value-of select="title"/></h2>
        <xsl:apply-templates select="title[1]/following-sibling::*"/>
     </p>
   </xsl:template>

   <xsl:template match="subsection">
     <p><h3>
        <xsl:number level="any" count="section" format="1."/>
        <xsl:number level="single" from="section" count="subsection" format="1."/>
        <a name="{@id}"/><xsl:value-of select="title"/></h3>
        <xsl:apply-templates select="title[1]/following-sibling::*"/>
     </p>
   </xsl:template>

   <xsl:template match="subsubsection">
     <p><h4>
        <xsl:number level="any" count="section" format="1."/>
        <xsl:number level="single" from="section" count="subsection" format="1."/>
        <xsl:number level="single" from="subsection" count="subsubsection" format="1."/>
        <a name="{@id}"/><xsl:value-of select="title"/></h4>
        <xsl:apply-templates select="title[1]/following-sibling::*"/>
     </p>
   </xsl:template>

   <xsl:template match="content/page/ROWSET">
     <p>
       <xsl:for-each select="ROW">
          <xsl:variable name="num1">
            <xsl:number level="any" count="section" format="1."/>
         </xsl:variable>
         <xsl:variable name="num2">
         <xsl:number level="single" from="section" count="subsection" format="1."/>
         </xsl:variable>
          <xsl:variable name="num3">
         <xsl:number level="single" from="subsection" count="subsubsection" format="1."/>
         </xsl:variable>         <p>
           <b><xsl:if test="$num1 !='0.'"><xsl:value-of select="$num1"/></xsl:if>
           <xsl:if test="$num2 !='0.'"><xsl:value-of select="$num2"/></xsl:if>
           <xsl:if test="$num3 !='0.'"><xsl:value-of select="$num3"/></xsl:if>
           <xsl:number level="single" count="ROW" format="1."/>  </b>

           <b><xsl:value-of disable-output-escaping="yes" select="XMLDATA/TITLE"/></b>
        </p>
        <xsl:if test="XMLDATA/QUESTION/text() !=''">
          <p>
            <b>Question:</b><br/><xsl:apply-templates select="XMLDATA/QUESTION"/>
          </p>
        </xsl:if>
         <p>
          <b>Answer:</b><br/><xsl:apply-templates select="XMLDATA/ANSWER"/>
        </p>
       </xsl:for-each>
       <a href="#top">back to top</a>
     </p>
   </xsl:template>
  
   <xsl:template match="table-of-contents">
      <h2>Table of Content</h2>
     <xsl:for-each select="/page/section[not(@id='content')]">
       <br/>
        <a href="#{@id}">
          <xsl:number level="any" count="section[not(@id='content')]" format="1."/>
           <xsl:text>0 </xsl:text>
          <xsl:value-of select="title"/>
        </a>
        <xsl:for-each select="subsection">
          <br/>&nbsp;&nbsp;
          <xsl:number level="single" from="page" count="section[not(@id='content')]" format="1."/> 
          <xsl:number level="single" from="section" count="subsection" format="1."/> 
             <a href="#{@id}"><xsl:value-of select="title"/></a>
        </xsl:for-each>
       </xsl:for-each>
    </xsl:template>
   

  <xsl:template match="KEYWORD">
    <xsl:variable name="content">
      <xsl:value-of select="./text()"/>
    </xsl:variable>
    <xsl:variable name="glossary">
      <xsl:value-of select="/page/GLOSSARY/ROWSET/ROW[NAME/text()=$content]/ID"/>
    </xsl:variable>
    <xsl:choose>
      <xsl:when test="$glossary">
        <a href="showGlossary.xsql?id={$glossary}">
          <xsl:value-of select="."/>
        </a>
      </xsl:when>
      <xsl:otherwise>
        <b>
          <xsl:value-of select="."/>
        </b>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  <xsl:template match="PARAGRAPH">
    <p>
      <xsl:apply-templates select="*|@*|comment()|processing-instruction()|text()"/>
    </p>
  </xsl:template>
   <xsl:template match="LINK">
    <a href="{@href}">
     <xsl:value-of select="."/>
    </a>
  </xsl:template>
  <xsl:template match="CODE">
    <pre>
      <xsl:value-of select="."/>
    </pre>
  </xsl:template>
  <xsl:template match="PARAGRAPH/CODE">
   <font class="code">
    <xsl:value-of select="."/>
  </font>
  </xsl:template>
  <!-- Identity transformation -->
  <xsl:template match="*|@*|comment()|processing-instruction()|text()">
    <xsl:copy>
      <xsl:apply-templates select="*|@*|comment()|processing-instruction()|text()"/>
    </xsl:copy>
  </xsl:template>  
</xsl:stylesheet>