---------------------------------------------------------------------------
-- DBCustomerSchema.sql
--
-- Create Database Schema for Customer Information
--
---------------------------------------------------------------------------
DROP TABLE app_product;
DROP TABLE pm_event;
drop table event_type;
drop table revenue_event;
DROP TABLE app_contact;
DROP TABLE app_occontact;
drop table oracle_contact;
drop table customer_app;
drop table contact;
drop table customer;

drop sequence customer_seq;
drop sequence contact_seq;
drop sequence oracle_contact_seq;
drop sequence customer_app_seq;

drop sequence pm_event_seq;
drop sequence revenue_event_seq;

---------------------------------------------------------------------------
-- Customer (status: interest, ongoing, reference)
---------------------------------------------------------------------------
CREATE SEQUENCE customer_seq START WITH 1;

create table customer
(
  customer_id NUMBER PRIMARY KEY,
  name VARCHAR2(100),
  description VARCHAR2(400),
  street VARCHAR2(100),
  city VARCHAR2(100),
  state VARCHAR2(100),
  zip VARCHAR2(100),
  country VARCHAR2(100),
  web_site VARCHAR2(200),
  status varchar2(20)
);

---------------------------------------------------------------------------
-- Create Tables and Sequences for Customer Applications and Contacts
---------------------------------------------------------------------------
CREATE SEQUENCE contact_seq START WITH 1;

CREATE TABLE contact
(
  contact_id NUMBER PRIMARY KEY,
  first_name VARCHAR2(100),
  mid_name VARCHAR2(10),
  last_name VARCHAR2(100),
  street VARCHAR2(100),
  city VARCHAR2(100),
  state VARCHAR2(100),
  zip VARCHAR2(100),
  country VARCHAR2(100),
  business_phone VARCHAR2(100),
  cell_phone VARCHAR2(100),
  email VARCHAR2(200),
  title VARCHAR2(50),
  customer_id NUMBER,
  constraint customer_contact_fk foreign key(customer_id) references customer(customer_id)
);

CREATE SEQUENCE oracle_contact_seq START WITH 1;

CREATE TABLE oracle_contact
(
  contact_id NUMBER PRIMARY KEY,
  first_name VARCHAR2(100),
  mid_name VARCHAR2(10),
  last_name VARCHAR2(100),
  street VARCHAR2(100),
  city VARCHAR2(100),
  state VARCHAR2(100),
  zip VARCHAR2(100),
  country VARCHAR2(100),
  business_phone VARCHAR2(100),
  cell_phone VARCHAR2(100),
  email VARCHAR2(200),
  department varchar2(200),
  title VARCHAR2(50)
);

CREATE SEQUENCE customer_app_seq START WITH 1;

CREATE TABLE customer_app
( 
  app_id NUMBER PRIMARY KEY,
  name VARCHAR2(200),
  app_type VARCHAR2(200),
  description VARCHAR2(4000),
  customer_id NUMBER,
  constraint customer_app_fk foreign key(customer_id) references customer(customer_id)
);

CREATE TABLE app_product
(
  app_id NUMBER,
  product_id VARCHAR2(50),
  CONSTRAINT app_id_fk FOREIGN KEY(app_id) REFERENCES customer_app(app_id),
  CONSTRAINT product_id_fk FOREIGN KEY(product_id) REFERENCES product(product_id)
); 

CREATE TABLE app_contact
(
  app_id NUMBER,
  customer_contact_id NUMBER,
  constraint app_fk foreign key(app_id) references customer_app(app_id),
  constraint app_ct_fk foreign key(customer_contact_id) references contact(contact_id)
); 

CREATE TABLE app_occontact
(
  app_id NUMBER,
  oracle_contact_id NUMBER,
  constraint app_oc_fk foreign key(app_id) references customer_app(app_id),
  constraint app_oct_fk foreign key(oracle_contact_id) references oracle_contact(contact_id)
); 

---------------------------------------------------------------------------
-- Create Tables and Sequences for PM Events 
---------------------------------------------------------------------------
CREATE TABLE event_type
(
  name VARCHAR2(100) PRIMARY KEY,
  description VARCHAR2(4000)
);

CREATE SEQUENCE pm_event_seq START WITH 1;

CREATE TABLE pm_event
(
  event_id number primary key,
  name VARCHAR2(100),
  type VARCHAR2(100),
  description VARCHAR2(4000),
  time date,
  customer_id number,
  constraint customer_pmevent_fk foreign key(customer_id) references customer(customer_id),
  constraint event_type_fk foreign key(type) references event_type(name)
);


CREATE SEQUENCE revenue_event_seq START WITH 1;

CREATE TABLE revenue_event
(
  event_id NUMBER PRIMARY KEY,
  name VARCHAR2(100),
  revenue_amount NUMBER,
  description VARCHAR2(4000),
  time DATE,
  customer_id NUMBER,
  CONSTRAINT customer_revent_fk FOREIGN KEY(customer_id) REFERENCES customer(customer_id)
);

-------------------------------------------------------------------------------
--  Collaterals (Category: presentation, whitepaper, tutorial)
-------------------------------------------------------------------------------
CREATE TABLE COLLATERAL(
 collateral_id  NUMBER PRIMARY KEY,
 title VARCHAR2(200),
 notes VARCHAR2(400),
 link VARCHAR2(200),
 createtime DATE NOT NULL,
 author VARCHAR2(150),
 category VARCHAR2(30)
);

CREATE SEQUENCE coll_seq START WITH 1;

CREATE or replace TRIGGER collateral_insert
BEFORE INSERT ON collateral
FOR EACH ROW
BEGIN
  select coll_seq.nextval into :new.collateral_id from dual;
END;
/
show errors;

---------------------------------------------------------------------------
PROMPT Create XML Product TAble ...
---------------------------------------------------------------------------
CREATE TABLE product
(
  product_id VARCHAR2(50) PRIMARY KEY,
  fullname VARCHAR2(100),
  parent_pd VARCHAR2(50),
  DESCRIPTION VARCHAR2(500)
);


