/**
 * File Name: XMLAnalysis.java
 * Description:
 *   Analysis XML using the annotated XML Schema 
 * Modified (MM/DD/YYYY)
 *   jiwang  04/15/2003 - Create
 * Notes:
 *  -  Using SAX Parser
 */
package oracle.xml.sample.semantics;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.File;
import java.io.Reader;
import java.io.Writer;
import java.io.PrintWriter;
import java.io.StringWriter;
import oracle.xml.parser.v2.SAXParser;
import oracle.xml.parser.schema.XMLSchema;
import oracle.xml.parser.schema.XSDBuilder;
import oracle.xml.parser.schema.XSDConstantValues;
import oracle.sql.CLOB;
import oracle.xml.parser.v2.DOMParser;
import oracle.xml.parser.v2.XMLDocument;
import java.io.StringReader;

public class XMLAnalysis {
  public XMLAnalysis() {
  }

  /**
   * Analyze XML based on SAX Parser
   * @param xml_doc Input XML document
   * @param xsd_doc Input XML Schema document
   * @exception XxxxxxException if ...
   * @return "OK" if success
   */
  public static String analyze(CLOB xml_doc, CLOB xsd_doc, CLOB[] res_doc) {
    Reader rd;
    Writer wt = null;

    // Check the Result CLOB
    if(res_doc[0] == null) {
      return "CLOB to write can not be null.";
    }
    try {
      wt =res_doc[0].getCharacterOutputStream();
      //Build up XML Schema Object
      rd = xsd_doc.getCharacterStream();
      XSDBuilder builder = new XSDBuilder();
      XMLSchema schemadoc = (XMLSchema)builder.build(rd,null);
      // Create Content Handlers 
      XMLAnalysisHandler POHandler = new XMLAnalysisHandler(new PrintWriter(wt));
      // Setup for SAX XML Schema Validation
      SAXParser parser = new SAXParser();
      parser.setContentHandler(POHandler);
      POHandler.setXMLProperty(XSDConstantValues.FIXED_SCHEMA,schemadoc);
      POHandler.setXMLProperty(XSDConstantValues.VALIDATION_MODE, 
                               XSDConstantValues.LAX_VALIDATION);
      // Parse the XML document
      rd = xml_doc.getCharacterStream();    
      parser.parse(rd);   
    } catch(Exception e) {
      e.printStackTrace();
      return e.getMessage();
    }
    return "OK";
  }
  /**
   * Main testing procedure
   */
  static public void main(String[] args) throws Exception  
  {
    String fileName;

    if(args.length != 2) {
      System.err.println("USAGE: java XMLAnalysis <schema file> <xml file>");
      return;
    }

    // Build Schema Object
    XSDBuilder builder = new XSDBuilder();
    URL    url =  createURL(args[0]);
    XMLSchema schemadoc = (XMLSchema)builder.build(url);
    StringWriter sw= new StringWriter();

    fileName = args[1];
    try {
      // Create handlers for the parser
      XMLAnalysisHandler POHandler = 
      new XMLAnalysisHandler(new PrintWriter(sw));

      // SAX Parsing with LAX mode XML Schema Validation
      SAXParser parser = new SAXParser();
      parser.setContentHandler(POHandler);
      POHandler.setXMLProperty(XSDConstantValues.FIXED_SCHEMA,schemadoc);
      POHandler.setXMLProperty(XSDConstantValues.VALIDATION_MODE, 
                               XSDConstantValues.LAX_VALIDATION);

      parser.parse(createURL(fileName));

      System.out.println(sw.toString());
    } 
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  static URL createURL(String fileName) {
    URL url = null;
    try {
      url = new URL(fileName);
    } catch(MalformedURLException ex) {
      File f = new File(fileName);
      try {
        String path = f.getAbsolutePath();
        // This is a bunch of weird code that is required to
        // make a valid URL on the Windows platform, due
        // to inconsistencies in what getAbsolutePath returns.
        String fs = System.getProperty("file.separator");
        if(fs.length() == 1) {
          char sep = fs.charAt(0);
          if(sep != '/')
            path = path.replace(sep, '/');
          if(path.charAt(0) != '/')
            path = '/' + path;
        }
        path = "file://" + path;
        url = new URL(path);
      } catch(MalformedURLException e) {
        System.out.println("Cannot create url for: " + fileName);
        System.exit(0);
      }
    }
    return url;
  }
}

