/**
 * POHandler.java
 * Description:
 *   SAX Content Handler for Purchase Order Semantic Processing
 * Modified (MM/DD/YYYY)
 *   jiwang  04/22/2003 - Add default XML element handling
 *   jiwang  04/14/2003 - Create
 */
package oracle.xml.sample.semantics;

import oracle.xml.parser.schema.XSDValidator;
import oracle.xml.parser.schema.XSDException;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.Hashtable;
import oracle.xml.parser.v2.XMLError;
import oracle.xml.parser.schema.XSDElement;
import oracle.xml.parser.schema.*;
import oracle.xml.parser.v2.*;

public class XMLAnalysisHandler extends XSDValidator {
  private PrintWriter out;
  private boolean isSelect;
  private StringBuffer buf;
  private int level;
  private Hashtable po_tbl;
  private Hashtable name_tbl;
  private DMElement cur_elem;
  private XMLError err;
  private XSDElement elementNode;

  public XMLAnalysisHandler(PrintWriter out) throws XSDException,SAXException
  {
    po_tbl = new Hashtable();
    name_tbl = new Hashtable();
    buf = new StringBuffer();
    err = new XMLError();
    setError(err);
    isSelect= false;
    level = 0;
    this.out= out;
    write("<PURCHASEORDER>\n");
  }

  public void startElement(String namespaceURI, String localName,
                           String qName, Attributes atts) throws SAXException
  {
    String schemaName = null;

    super.startElement(namespaceURI,localName,qName,atts);
    // Create the DMElement at the start of LAX validation
    //if (!state.isFake() && level == 0) {
    if(getCurrentMode()==_strict && level==0)
    {
      isSelect=true;
      cur_elem= new DMElement();
      //level=curState;
      level=0;
    }
    if (isSelect == true) {
      level++;
       //Store XML Element Content
      String elementName = getElementName(localName);
      if(elementName != null)
      {
       name_tbl.put(localName, elementName);
       writeElement("<"+elementName+">");
       printAttributes(atts);
      }
    }
  }

  public void endElement(String namespaceURI, String localName,String qName) 
  throws SAXException
  {
    XSDAnnotation ann;
    String elementName;

    super.endElement(namespaceURI, localName,qName);   

    if (isSelect == true) {
      level--;
      flushChar();
      // Output the endTag of the Element
      elementName = (String)name_tbl.get(localName);
      writeElement("</"+elementName);
      writeElement(">");      
      // If validation errors exists, exit the process
      int i = err.getFirstError();
      if (i!=-1)
        throw new 
        SAXException("(Line,Column)("+err.getLineNumber(i)+","+
                     err.getColumnNumber(i)+"):"+err.getMessage(i));
      // Save the Validated Element
      cur_elem.level = level;
      cur_elem.isValid=true;
      po_tbl.put(elementName, cur_elem);
      //System.out.println(cur_elem.content);
      //if (level > curState) {
      if(level==0){
       // level =0;
        isSelect=false;
      }
    } else {
      flushChar();
    }
  }

  public void characters(char ch[], int start, int length) throws SAXException
  {
    // Normalization
    String str = new String(ch, start, length);
    str = str.replace('\t',' ');
    str = str.replace('\n',' ');
    str = str.replace('\r',' ');
    str = str.trim();
    char[] ca = str.toCharArray();
    int i, j;
    boolean seenWS = false;
    for (i=0,j=0; j< str.length(); j++) {
      if (ca[j] != ' ' || !seenWS) {
        ca[i++] = ca[j];
        if (ca[j] == ' ')
          seenWS = true;
        else
          seenWS = false;
      }
    }         
    buf.append(ca,0,i);
  }

  public void endDocument() throws SAXException
  {
    String[] ship_elems = new String[]{"SHIPPING_ADDR"};
    String[] bill_elems = new String[]{"BILLING_ADDR"};
    String[] sendby_elems = new String[]{"SENDBY_ADDR"};
    String[] line_elems = new String[]{"LINEITEMS"};
    DMElement ck_elem = null;

    // Add sendby Address
    for (int i=0; i<1; i++) {
      ck_elem = (DMElement) po_tbl.get(sendby_elems[i]);
      if (ck_elem != null)
        if (ck_elem.isValid == true) {
          write(ck_elem.content);
          break;
        }
    }   
    if (ck_elem == null || ck_elem.content == null)
    {
      write("<SENDBY_ADDR opt=\"default\"/>");
    }
    else if (ck_elem.isValid = false)
      throw new SAXException("Sendby Address is not Valid!");

    //Check Required Shipping Address 
    for (int i=0; i<1; i++) {
      ck_elem = (DMElement) po_tbl.get(ship_elems[i]);
      if (ck_elem != null)
        if (ck_elem.isValid == true) {
          write(ck_elem.content);
          break;
        }
    }   
    if (ck_elem == null)
      throw new SAXException("Miss required Shipping Address!");
    else if (ck_elem.content == null)
      throw new SAXException("Shipping Address dosen't have content!");
    else if (ck_elem.isValid = false)
      throw new SAXException("Shipping Address is not Valid!");

    //Check Required Billing Address 
    ck_elem = null;
    for (int i=0; i<1; i++) {
      ck_elem = (DMElement) po_tbl.get(bill_elems[i]);
      if (ck_elem != null)
        if (ck_elem.isValid == true) {
          write(ck_elem.content);
          break;
        }
    }   
    if (ck_elem == null)
      throw new SAXException("Miss required Billing Address!");
    else if (ck_elem.content == null)
      throw new SAXException("Billing Address doesn't have content!");
    else if (ck_elem.isValid == false)
      throw new SAXException("Billing Address is not Valid!");

    //Check Required LineItems 
    ck_elem = null;
    for (int i=0; i<1; i++) {
      ck_elem = (DMElement) po_tbl.get(line_elems[i]);
      if (ck_elem != null)
        if (ck_elem.isValid == true) {
          write(ck_elem.content);
          break;
        }
    }   
    if (ck_elem == null)
      throw new SAXException("Miss required LineItems!");
    else if (ck_elem.content == null)
      throw new SAXException("LineItems doesn't have content!");
    else if (ck_elem.isValid == false)
      throw new SAXException("LineItems is not Valid!");

    write("</PURCHASEORDER>");
    out.flush();
  }

  private void write(String s) {
    out.write(s); 
  }

  private void writeElement(String s) {
    if (isSelect == true && cur_elem != null)
      cur_elem.write(s);
  }

  private void flushChar() {
    if (buf.length() >0) {
      writeElement(buf.toString());
      buf.setLength(0);
    }
  }

  /*
   * Print out Attribute into XMLElements
   */
  void printAttributes(Attributes atts) {
    String attr_name;

    int len = atts.getLength();
    if (len <1) return;
    for (int i=0; i< len; i++) {
      attr_name = getAttributeName(atts.getLocalName(i));  
    if(attr_name != null)
    {
      writeElement("<"+attr_name+">");
      writeElement(atts.getValue(i));
      writeElement("</"+attr_name+">");
    }
    }
  }

  /**
   * Get Attribute Name for XML Schema Annotations
   */
  String getAttributeName(String localName) {
    String attrName="";
    if(elementNode == null) return localName;

    ///////////////////////////////////////////////////////////////////////////
    // Notes:
    //   If user use xsi:type in the instance doc the we need to use this:
    //    XSDNode currNode = getTypeDefinition();
    //   If(currNode.getNodeType == DATATYPE) return;
    //   If currNode.getNodeType == TYPE
    //    We can get attributes by:
    //    XSDAttribute[] attrNode = currNode.getAttributeDeclarations();
    elementNode = getElementDeclaration();
    XSDAttribute[] attrNode = elementNode.getAttributeDeclarations();

    for (int j=0;j<attrNode.length; j++) {
      if (attrNode[j].getName().equals(localName)) {
        XSDAnnotation ann = attrNode[j].getAnnotation();
        if ( ann != null) {
          Vector attrs = ann.getAttributes();
          int attrs_size = attrs.size();
          for (int i =0; i< attrs_size; i++) {
            XMLAttr xsd_attr = (XMLAttr)attrs.get(i);
            String name = xsd_attr.getName();
            if (name.equals("xdk:SQLName")) {
              attrName= xsd_attr.getValue();
              break;
            }
          }
        }
       }
    }
    if (attrName.equals("")) return null;  //attrName = localName;

    return attrName;
  }
  /**
   * Get Element Name from the XML Schema Annotations or Substituion Group
   * Definitions
   */
  String getElementName(String localName) {
    String elementName=""; 
    elementNode = getElementDeclaration();
    
    // XDK Annotations 
    if (elementNode != null) {
      XSDAnnotation ann = elementNode.getAnnotation();
      if ( ann != null) {
        Vector attrs = ann.getAttributes();
        int attrs_size = attrs.size();
        for (int i =0; i< attrs_size; i++) {
          XMLAttr xsd_attr = (XMLAttr)attrs.get(i);
          String name = xsd_attr.getName();
          if (name.equals("xdk:SQLName")) {
            elementName= xsd_attr.getValue();
            break;
          }
        }
      }
      if (elementName.equals("")) return null; //elementName=localName;
    }
    return elementName;
  }

}
/**
 * Data Mining Element
 */
class DMElement {
  public int level = 0;
  public boolean isValid = false;  
  public String content = "";
  public String elementName;

  public void write(String in) {
    content += in;
  }
}
