-------------------------------------------------------------------------------
-- FileName: createObjectView.sql
-- Description: 
--   Create Relational Table, Object View for Data Insertion
-- Modified (MM/DD/YYYY)
--   jiwang  04/15/2003 - Create
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--  Create Object-Relational Tables
-------------------------------------------------------------------------------
set echo on
drop table po;
drop table lineitem;
drop table lineitems;
drop sequence poobj_seq;
drop table poid;

CREATE TABLE poid(
  current_id raw(16)
);

CREATE TABLE PO(
  po_id raw(16) PRIMARY KEY,
  sendby_csname VARCHAR2(50),
  sendby_street VARCHAR2(100),
  sendby_city VARCHAR2(200),
  sendby_state VARCHAR2(20),
  sendby_zip VARCHAR2(20),
  sendby_country VARCHAR2(30),
  shipping_csname VARCHAR2(50),
  shipping_street VARCHAR2(100),
  shipping_city VARCHAR2(200),
  shipping_state VARCHAR2(20),
  shipping_zip VARCHAR2(20),
  shipping_country VARCHAR2(30),
  billing_csname VARCHAR2(50),
  billing_street VARCHAR2(100),
  billing_city VARCHAR2(200),
  billing_state VARCHAR2(20),
  billing_zip VARCHAR2(20),
  billing_country VARCHAR2(30)
);

CREATE TABLE LineItem(
  product_id VARCHAR2(10) primary key,
  product_name VARCHAR2(10),
  product_quantity NUMBER(30),
  product_price VARCHAR2(20)
);

CREATE TABLE LineItems(
  po_id raw(16),
  product_id VARCHAR2(10) 
);

CREATE SEQUENCE poobj_seq START WITH 1;

-------------------------------------------------------------------------------
--  Create Object View
-------------------------------------------------------------------------------
drop type po_typ;
drop type address_typ;
drop type lineitem_list;
drop type lineitem_typ;

CREATE TYPE address_typ AS OBJECT(
  customer_name VARCHAR2(50),
  street  VARCHAR2(100),
  city VARCHAR2(200),
  state VARCHAR2(20),
  zip VARCHAR2(20),
  country VARCHAR2(30)
);
/
CREATE TYPE lineitem_typ AS OBJECT(
  product_id VARCHAR2(10),
  product_name VARCHAR2(10),
  product_quantity NUMBER(30),
  product_price VARCHAR2(20)
);
/
CREATE TYPE lineitem_list AS TABLE OF lineitem_typ;
/
CREATE TYPE po_typ AS OBJECT(
 po_id raw(16),
 sendby_addr address_typ,
 shipping_addr address_typ,
 billing_addr address_typ,
 lineitems lineitem_list
);
/
CREATE OR REPLACE VIEW purchaseorder OF po_typ
WITH OBJECT OID (po_id) AS
  select po_id, address_typ(sendby_csname,
                              sendby_street,
                              sendby_city,
                              sendby_state,
                              sendby_zip,
                              sendby_country), 
                 address_typ(shipping_csname,
                              shipping_street,
                              shipping_city,
                              shipping_state,
                              shipping_zip,
                              shipping_country), 
                  address_typ(billing_csname,
                              billing_street,
                              billing_city,
                              billing_state,
                              billing_zip,
                              billing_country),                  
                 cast(multiset(
                               select lineitem.product_id, product_name, 
                               product_quantity, product_price                              
                               from lineitem, lineitems
                               where lineitems.po_id=po.po_id and
                                     lineitem.product_id=lineitems.product_id
                               ) as lineitem_list 
                      ) lineitems
     from po;
-------------------------------------------------------------------------------
--  Create Instead of Trigger 
-------------------------------------------------------------------------------
DROP TRIGGER purchaseorder_insert_trigger;
CREATE TRIGGER purchaseorder_insert_trigger
INSTEAD OF INSERT ON purchaseorder
FOR EACH ROW 
DECLARE
  v_lineitems lineitem_list;
  v_lineitem  lineitem_typ;
  v_id RAW(16);
  v_num number;
BEGIN
   ----------------------------------------------------------------------------
   -- Insert into PO table
   ----------------------------------------------------------------------------
   SELECT current_id into v_id from poid  where rownum=1;
   INSERT INTO po 
     values (v_id,
                 :new.sendby_addr.customer_name,
                 :new.sendby_addr.street, 
                 :new.sendby_addr.city,
                 :new.sendby_addr.state,
                 :new.sendby_addr.zip,
                 :new.sendby_addr.country,
                 :new.shipping_addr.customer_name,
                 :new.shipping_addr.street, 
                 :new.shipping_addr.city,
                 :new.shipping_addr.state,
                 :new.shipping_addr.zip,
                 :new.shipping_addr.country,
                 :new.billing_addr.customer_name,
                 :new.billing_addr.street, 
                 :new.billing_addr.city,
                 :new.billing_addr.state,
                 :new.billing_addr.zip,
                 :new.billing_addr.country);
   ----------------------------------------------------------------------------
   -- Insert into LineItem and LineItems table
   ----------------------------------------------------------------------------
   v_lineitems := :new.lineitems;
   for i in 1..v_lineitems.count loop
     v_lineitem := v_lineitems(i);
     select count(1) into v_num from lineitem 
     where product_id = v_lineitem.product_id;
     if v_num = 0 then
     insert into lineitem
       values(v_lineitem.product_id, 
              v_lineitem.product_name,
              v_lineitem.product_quantity,
              v_lineitem.product_price);
      end if;
     insert into lineitems values(v_id, v_lineitem.product_id);
   end loop;
end;
/
show error;

-------------------------------------------------------------------------------
--  Grant read/write privilege to POADMIN
-------------------------------------------------------------------------------
grant SELECT,DELETE on po to poadmin;
grant SELECT,DELETE  on lineitem to poadmin;
grant SELECT,DELETE  on lineitems to poadmin;
grant SELECT,DELETE  on poid to poadmin;

