-------------------------------------------------------------------------------
-- FileName: createXMLProcedures.sql
-- Description: 
--   Create XML Procedure for Dequeuing Messages using XML Semantic Analysis
-- Modified (MM/DD/YYYY)
--   jiwang  04/23/2003 - Add Long Name to Short Name Mapping with 
--                        createMappingTable.sql
--   jiwang  04/22/2003 - Add Optional Element Processing
--                        with createPOUserTable.sql
--   jiwang  04/21/2003 - Add Batch Process
--   jiwang  04/15/2003 - Create
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- XMLSemanticProcessing Function (Java Stored Procedure)
-------------------------------------------------------------------------------
CREATE OR REPLACE FUNCTION XMLSemanticProcessing(xml IN CLOB,
                                                 xsd IN CLOB,
                                                 result IN OUT CLOB)
RETURN VARCHAR2
IS LANGUAGE JAVA NAME 
'oracle.xml.sample.semantics.XMLAnalysis.analyze(oracle.sql.CLOB,
                  oracle.sql.CLOB,oracle.sql.CLOB[]) returns java.lang.String';
/
show errors;

-------------------------------------------------------------------------------
-- printClobOut Procedure
-------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE printClobOut(result IN OUT NOCOPY CLOB) is
  str varchar2(32767);
  tmpstr varchar2(32767);
  len integer;
  idx integer := 1;
  nlidx integer;
  new_line varchar2(2) := '
';
  begin
      str := dbms_lob.SUBSTR(result,31*1024);

    len := length(str);
    loop
      exit when idx >len;
      nlidx := instr(str,new_line,idx);
      if (nlidx = 0) then
         nlidx := len+1;
      end if;
      tmpstr := substr(str,idx,nlidx-idx);
     
      if(length(tmpstr) > 250) then 
        --print line or 250 bytes per line
        tmpstr := '| '||tmpstr;
        loop
          exit when tmpstr is null;
          dbms_output.put_line(substr(tmpstr,1,250));
          tmpstr := substr(tmpstr,251);
        end loop;
      else
        dbms_output.put_line('| '||tmpstr);
      end if; 
      
      idx := nlidx+1;
    end loop;
end;
/
-------------------------------------------------------------------------------
-- Insert the XML Docuemnt using XSU 
-------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE insProc(xmlDoc IN CLOB, tableName IN VARCHAR2) is
   insCtx DBMS_XMLSave.ctxType;
   rows NUMBER;
 BEGIN
    insCtx := DBMS_XMLSave.newContext(tableName); -- get the context handle
    DBMS_XMLSave.setRowtag(insCtx,'PURCHASEORDER');
    rows := DBMS_XMLSave.insertXML(insCtx,xmlDoc); -- this inserts the document
    DBMS_OUTPUT.put_line(rows||' rows inserted');
    DBMS_XMLSave.closeContext(insCtx);            -- this closes the handle
END;
/


-------------------------------------------------------------------------------
-- Dequeue XML Messages Procedure with XML Analysis 
-------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE dequeueXMLAnalysis(p_consumer IN VARCHAR2,
                                                p_queue IN VARCHAR2) 
   AS
    v_msgid           RAW(16);
    v_dopt            dbms_aq.dequeue_options_t;
    v_mprop           dbms_aq.message_properties_t;
    v_payload         XMLTYPE;
    no_messages       EXCEPTION;
    PRAGMA EXCEPTION_INIT(no_messages, -25228);
    v_xml   clob;
    v_xsd   CLOB;
    v_result CLOB;
    v_insert CLOB;
    v_out VARCHAR2(32767);
    v_temp xmltype;
    v_sendby VARCHAR2(32767);
    v_id VARCHAR2(10);
    v_i NUMBER;
    v_name VARCHAR2(100);
    v_doc XMLTYPE;
BEGIN
   ----------------------------------------------------------------------------
   -- Get XML Document
   ----------------------------------------------------------------------------
   -- Setup Dequeue Options
   v_dopt.consumer_name := p_consumer;
   v_dopt.dequeue_mode := DBMS_AQ.REMOVE;
   v_dopt.wait := DBMS_AQ.NO_WAIT;
   v_dopt.navigation := DBMS_AQ.FIRST_MESSAGE;
   -----------------------------------------------------------------------------
   --  Dequeue All the XML Message in the Message Queue
   -----------------------------------------------------------------------------
   LOOP
     DBMS_AQ.dequeue(queue_name => p_queue,
                    dequeue_options => v_dopt,
                    message_properties => v_mprop,
                    payload => v_payload,
                    msgid => v_msgid);
     v_xml := v_payload.getClobVal();
     printClobOut(v_xml);
     ---------------------------------------------------------------------------
     -- Get XML Schema Document
     ---------------------------------------------------------------------------
     SELECT xsd INTO v_xsd FROM xsd_tbl WHERE id=1;
     printClobOut(v_xsd);
     ---------------------------------------------------------------------------
     -- Analyze the XML Document
     ---------------------------------------------------------------------------
     DBMS_LOB.createtemporary(v_result,TRUE,DBMS_LOB.SESSION);
     DBMS_LOB.createtemporary(v_insert,TRUE,DBMS_LOB.SESSION);
     v_out := XMLSemanticProcessing(v_xml,v_xsd,v_result);
     IF v_out = 'OK' THEN
       INSERT INTO po_tbl(msgid, consumer, podoc)
        VALUES(v_msgid,p_consumer, XMLTYPE(v_result));
       INSERT INTO po_backup_tbl(msgid, consumer, podoc)
        VALUES(v_msgid,p_consumer, XMLTYPE(v_xml));
       -------------------------------------------------------------------------
       -- Process Optional Element
       -------------------------------------------------------------------------
       v_temp := XMLTYPE(v_result);
       IF v_temp.existsNode('/PURCHASEORDER/SENDBY_ADDR[@opt="default"]')>0 THEN 
         v_id := v_mprop.sender_id.name;
         SELECT updateXML(v_temp,
            '/PURCHASEORDER/SENDBY_ADDR',
            (SELECT XMLElement("SENDBY_ADDR",
               XMLFOREST(
                 name as "CUSTOMER_NAME",
                 street as "STREET",
                 city as "CITY",
                 state as "STATE",
                 zip as "ZIP",
                 country as "COUNTRY")) 
               FROM POUSER 
               WHERE user_id = v_mprop.sender_id.name)
          ).getClobVal() INTO v_result 
         FROM dual;
       END IF;
       printClobOut(v_result);
       -------------------------------------------------------------------------
       -- Name Mapping for Product Names
       -------------------------------------------------------------------------
       v_doc := XMLTYPE(v_result);
       IF v_doc.existsNode('/PURCHASEORDER/LINEITEMS//PRODUCT_NAME') = 1 THEN
          v_i:=1;

          LOOP
            v_temp:= 
            v_doc.extract('/PURCHASEORDER/LINEITEMS/LINEITEM['||v_i||']/PRODUCT_NAME/text()');
            EXIT WHEN v_temp IS NULL;

            -- Get the mapping element
            SELECT short_name INTO v_name FROM mapping
            WHERE upper(long_name) = upper(v_temp.getStringVal());

            SELECT updateXML(v_doc,
                    '/PURCHASEORDER/LINEITEMS/LINEITEM['||v_i||']/PRODUCT_NAME/text()',
                     v_name) INTO v_doc     
            FROM dual;
             v_i := v_i+1;
           END LOOP;
      END IF;
      SELECT v_doc.getClobVal() INTO v_result FROM dual;
       -------------------------------------------------------------------------
       -- Insert the XML Documents using XSU
       -------------------------------------------------------------------------
       DBMS_LOB.OPEN(v_insert, DBMS_LOB.LOB_READWRITE);
       DBMS_LOB.writeappend(v_insert,length('<ROWSET>'),'<ROWSET>');
       DBMS_LOB.append(v_insert,v_result);
       DBMS_LOB.writeappend(v_insert,length('</ROWSET>'),'</ROWSET>');
       DBMS_LOB.CLOSE(v_insert);
       dbms_output.put_line(v_msgid);
       printClobOut(v_insert);
       INSERT INTO poid values(v_msgid);
       insProc(v_insert,'purchaseorder');
       printClobOut(v_insert);
   
       DBMS_LOB.freetemporary(v_insert);
       DBMS_LOB.freetemporary(v_result);
       -- Commit the transaction
       DELETE FROM poid;        
     ELSE
       DBMS_OUTPUT.put_line(v_out);
       INSERT INTO po_rejected_tbl(msgid, consumer, podoc,reason)
          VALUES(v_msgid,p_consumer, XMLTYPE(v_xml),v_out);
     END IF;
     COMMIT;
     v_dopt.navigation := DBMS_AQ.NEXT_MESSAGE;
   END LOOP;
   
EXCEPTION
    WHEN no_messages THEN
      dbms_output.put_line( 'Error:'||'No more messages in queue.');
END;
/
show errors;

-------------------------------------------------------------------------------
-- Dequeue XML Messages Procedure with XML Analysis 
-------------------------------------------------------------------------------
CREATE OR REPLACE PROCEDURE dequeueXMLAnalysisDemo(p_consumer IN VARCHAR2,
                                                p_queue IN VARCHAR2) 
   AS
    v_msgid           RAW(16);
    v_dopt            dbms_aq.dequeue_options_t;
    v_mprop           dbms_aq.message_properties_t;
    v_payload         XMLTYPE;
    no_messages       EXCEPTION;
    PRAGMA EXCEPTION_INIT(no_messages, -25228);
    v_xml   clob;
    v_xsd   CLOB;
    v_result CLOB;
    v_insert CLOB;
    v_out VARCHAR2(32767);
BEGIN
   ----------------------------------------------------------------------------
   -- Get XML Document
   ----------------------------------------------------------------------------
   -- Setup Dequeue Options
   v_dopt.consumer_name := p_consumer;
   v_dopt.dequeue_mode := DBMS_AQ.REMOVE;
   v_dopt.wait := DBMS_AQ.NO_WAIT;
   v_dopt.navigation := DBMS_AQ.FIRST_MESSAGE;
   -- Dequeue XML Message
   DBMS_AQ.dequeue( queue_name => p_queue,
                    dequeue_options => v_dopt,
                    message_properties => v_mprop,
                    payload => v_payload,
                    msgid => v_msgid);
   v_xml := v_payload.getClobVal();
   printClobOut(v_xml);
   ----------------------------------------------------------------------------
   -- Get XML Schema Document
   ----------------------------------------------------------------------------
   SELECT xsd INTO v_xsd FROM xsd_tbl WHERE id=1;
   printClobOut(v_xsd);
   ----------------------------------------------------------------------------
   -- Analyze the XML Document
   ----------------------------------------------------------------------------
   DBMS_LOB.createtemporary(v_result,TRUE,DBMS_LOB.SESSION);
   DBMS_LOB.createtemporary(v_insert,TRUE,DBMS_LOB.SESSION);
   v_out := XMLSemanticProcessing(v_xml,v_xsd,v_result);
   IF v_out = 'OK' THEN
     printClobOut(v_result);
     INSERT INTO po_tbl(msgid, consumer, podoc)
        VALUES(v_msgid,p_consumer, XMLTYPE(v_result));
     INSERT INTO po_backup_tbl(msgid, consumer, podoc)
        VALUES(v_msgid,p_consumer, XMLTYPE(v_xml));
     --------------------------------------------------------------------------
     -- Insert the XML Documents using XSU
     --------------------------------------------------------------------------
     DBMS_LOB.OPEN(v_insert, DBMS_LOB.LOB_READWRITE);
     DBMS_LOB.writeappend(v_insert,length('<ROWSET>'),'<ROWSET>');
     DBMS_LOB.append(v_insert,v_result);
     DBMS_LOB.writeappend(v_insert,length('</ROWSET>'),'</ROWSET>');
     DBMS_LOB.CLOSE(v_insert);
     INSERT INTO poid values(v_msgid);
     insProc(v_insert,'purchaseorder');
     printClobOut(v_insert);
   
     DBMS_LOB.freetemporary(v_insert);
     DBMS_LOB.freetemporary(v_result);
     -- Commit the transaction
     DELETE FROM poid;        
  ELSE
       DBMS_OUTPUT.put_line(v_out);
       INSERT INTO po_rejected_tbl(msgid, consumer, podoc,reason)
          VALUES(v_msgid,p_consumer, XMLTYPE(v_xml),v_out);
  END IF;
  COMMIT;
  
EXCEPTION
    WHEN no_messages THEN
      dbms_output.put_line( 'Error:'||'No more messages in queue.');
      commit;
END;
/
show errors;


-- exec dequeueXMLAnalysis('scott','poadmin.poxml_q');
-- Quit

