-------------------------------------------------------------------------------
-- FileName: testUpdateXML.sql
-- Description: 
--   Update XML in PL/SQL procedures
-- Modified (MM/DD/YYYY)
--   jiwang  04/23/2003 - Create
-------------------------------------------------------------------------------
set echo on
SET autoprint ON
var v_result CLOB;
DECLARE
   v_doc XMLTYPE := XMLTYPE('<PURCHASEORDER><SENDBY_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Nancy Rendall</CUSTOMER_NAME><STREET>One Alewife Center</STREET><CITY>Cambridge</CITY><STATE>MA</STATE><ZIP>02140</ZIP></SENDBY_ADDR><SHIPPING_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Bob White</CUSTOMER_NAME><STREET>567 Maple Parkway</STREET><CITY>Mill Valley</CITY><STATE>CA</STATE><ZIP>90976</ZIP></SHIPPING_ADDR><BILLING_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Joseph Smith</CUSTOMER_NAME><STREET>8 Oak Avenue</STREET><CITY>Old Town</CITY><STATE>PA</STATE><ZIP>94320</ZIP></BILLING_ADDR><LINEITEMS><LINEITEM><PRODUCT_ID>874-AC</PRODUCT_ID><PRODUCT_NAME>Dell Laptop</PRODUCT_NAME><PRODUCT_QUANTITY>1</PRODUCT_QUANTITY><PRODUCT_PRICE>1480.95</PRODUCT_PRICE></LINEITEM><LINEITEM><PRODUCT_ID>956-DD</PRODUCT_ID><PRODUCT_NAME>Disk Washer</PRODUCT_NAME><PRODUCT_QUANTITY>2</PRODUCT_QUANTITY><PRODUCT_PRICE>339.98</PRODUCT_PRICE></LINEITEM></LINEITEMS></PURCHASEORDER>');
   v_temp XMLTYPE;
BEGIN
   -- Simple Update
   SELECT updateXML(v_doc,'/PURCHASEORDER/SENDBY_ADDR',
                   '<SENDBY_ADDR/>').getClobVal() INTO  :v_result FROM dual;

   -- Update from SQL/XML functions
   SELECT updateXML(v_doc,
            '/PURCHASEORDER/SENDBY_ADDR',
            (SELECT XMLElement("SENDBY_ADDR",
               XMLFOREST(
                 name as "SENDBY_CSNAME",
                 street as "SENDBY_STREET",
                 city as "SENDBY_CITY",
                 state as "SENDBY_STATE",
                 zip as "SENDBY_ZIP",
                 country as "SENDBY_COUNTRY")) from POUSER where user_id = 'bob')
                 ).getClobVal() INTO :v_result FROM dual;
end;
-------------------------------------------------------------------------------
--  Update the Product Names
-------------------------------------------------------------------------------
set echo on
SET autoprint ON
var v_result CLOB;
DECLARE
   v_doc XMLTYPE := XMLTYPE('<PURCHASEORDER><SENDBY_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Nancy Rendall</CUSTOMER_NAME><STREET>One Alewife Center</STREET><CITY>Cambridge</CITY><STATE>MA</STATE><ZIP>02140</ZIP></SENDBY_ADDR><SHIPPING_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Bob White</CUSTOMER_NAME><STREET>567 Maple Parkway</STREET><CITY>Mill Valley</CITY><STATE>CA</STATE><ZIP>90976</ZIP></SHIPPING_ADDR><BILLING_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Joseph Smith</CUSTOMER_NAME><STREET>8 Oak Avenue</STREET><CITY>Old Town</CITY><STATE>PA</STATE><ZIP>94320</ZIP></BILLING_ADDR><LINEITEMS><LINEITEM><PRODUCT_ID>874-AC</PRODUCT_ID><PRODUCT_NAME>Lawnmower</PRODUCT_NAME><PRODUCT_QUANTITY>1</PRODUCT_QUANTITY><PRODUCT_PRICE>1480.95</PRODUCT_PRICE></LINEITEM><LINEITEM><PRODUCT_ID>956-DD</PRODUCT_ID><PRODUCT_NAME>Baby Monitor</PRODUCT_NAME><PRODUCT_QUANTITY>2</PRODUCT_QUANTITY><PRODUCT_PRICE>339.98</PRODUCT_PRICE></LINEITEM></LINEITEMS></PURCHASEORDER>');
   v_temp XMLTYPE;
   v_name varchar2(100);
   i number;
BEGIN
   -- Long Name to Short Name Mapping
   IF v_doc.existsNode('/PURCHASEORDER/LINEITEMS//PRODUCT_NAME') = 1 THEN
     dbms_output.put_line('OK');
      i:=1;

      LOOP
        v_temp := 
        v_doc.extract('/PURCHASEORDER/LINEITEMS/LINEITEM['||i||']/PRODUCT_NAME/text()');
        EXIT WHEN v_temp IS NULL;
        dbms_output.put_line(v_temp.getStringVal());
        select short_name into v_name from mapping
        where upper(long_name) = upper(v_temp.getStringVal());

        SELECT updateXML(v_doc,
                '/PURCHASEORDER/LINEITEMS/LINEITEM['||i||']/PRODUCT_NAME/text()',
                 v_name).getClobVal() into v_doc        
        FROM dual;
        dbms_output.put_line(v_name);
         i := i+1;
        COMMIT;
       END LOOP;
       SELECT v_doc.getStringVal() INTO :v_result FROM dual;
  END IF;
END;
/
show errors;

-------------------------------------------------------------------------------
--  Batch Update
-------------------------------------------------------------------------------
CREATE TABLE temp(doc xmltype);
INSERT INTO temp 
  VALUES(XMLTYPE('<PURCHASEORDER><SENDBY_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Nancy Rendall</CUSTOMER_NAME><STREET>One Alewife Center</STREET><CITY>Cambridge</CITY><STATE>MA</STATE><ZIP>02140</ZIP></SENDBY_ADDR><SHIPPING_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Bob White</CUSTOMER_NAME><STREET>567 Maple Parkway</STREET><CITY>Mill Valley</CITY><STATE>CA</STATE><ZIP>90976</ZIP></SHIPPING_ADDR><BILLING_ADDR><COUNTRY>US</COUNTRY><CUSTOMER_NAME>Joseph Smith</CUSTOMER_NAME><STREET>8 Oak Avenue</STREET><CITY>Old Town</CITY><STATE>PA</STATE><ZIP>94320</ZIP></BILLING_ADDR><LINEITEMS><LINEITEM><PRODUCT_ID>874-AC</PRODUCT_ID><PRODUCT_NAME>Baby Monitor</PRODUCT_NAME><PRODUCT_QUANTITY>1</PRODUCT_QUANTITY><PRODUCT_PRICE>1480.95</PRODUCT_PRICE></LINEITEM><LINEITEM><PRODUCT_ID>956-DD</PRODUCT_ID><PRODUCT_NAME>Lawnmower</PRODUCT_NAME><PRODUCT_QUANTITY>2</PRODUCT_QUANTITY><PRODUCT_PRICE>339.98</PRODUCT_PRICE></LINEITEM></LINEITEMS></PURCHASEORDER>'));

SELECT updateXML(value(p),'/PRODUCT_NAME/text()', m.short_name)
FROM temp q, 
   TABLE(XMLSEQUENCE(extract(q.doc,'/PURCHASEORDER/LINEITEMS/LINEITEM/PRODUCT_NAME'))) p,
   mapping m
where m.long_name = extractValue(value(p),'/PRODUCT_NAME/text()');

SELECT updateXML(q.doc.extract('/PURCHASEORDER/LINEITEMS/LINEITEM/PRODUCT_NAME[text()="'||
       extractValue(value(p),'/PRODUCT_NAME/text()')||'"]'),'/PRODUCT_NAME/text()',
       m.short_name)
FROM temp q, 
   TABLE(XMLSEQUENCE(extract(q.doc,'/PURCHASEORDER/LINEITEMS/LINEITEM/PRODUCT_NAME'))) p,
   mapping m
where m.long_name = extractValue(value(p),'/PRODUCT_NAME/text()');
