-------------------------------------------------------------------------------
-- FileName: testXMLTypeView Usage
-- Description: 
--   Test date query and insersion to XMLType views 
-- Modified (MM/DD/YYYY)
--   jiwang  04/25/2003 - Add the PL/SQL Procedure for PO Insersion
--   jiwang  04/16/2003 - Create
-- Note: Pending to finish.
-------------------------------------------------------------------------------
set echo on
drop table po;
drop table lineitem;
drop table lineitems;
drop sequence poobj_seq;
drop table poid;

CREATE TABLE poid(
  current_id raw(16)
);

CREATE TABLE PO(
  po_id raw(16) PRIMARY KEY,
  shipping_csname VARCHAR2(50),
  shipping_street VARCHAR2(100),
  shipping_city VARCHAR2(200),
  shipping_state VARCHAR2(20),
  shipping_zip VARCHAR2(20),
  shipping_country VARCHAR2(30),
  billing_csname VARCHAR2(50),
  billing_street VARCHAR2(100),
  billing_city VARCHAR2(200),
  billing_state VARCHAR2(20),
  billing_zip VARCHAR2(20),
  billing_country VARCHAR2(30)
);

CREATE TABLE LineItem(
  product_id VARCHAR2(10),
  product_name VARCHAR2(30),
  product_quantity NUMBER(30),
  product_price VARCHAR2(20)
);

CREATE TABLE LineItems(
  po_id raw(16),
  product_id VARCHAR2(10) 
);
-------------------------------------------------------------------------------
--  Create Object View
-------------------------------------------------------------------------------
drop type po_typ;
drop type address_typ;
drop type lineitem_list;
drop type lineitem_typ;

CREATE TYPE address_typ AS OBJECT(
  customer_name VARCHAR2(50),
  street  VARCHAR2(100),
  city VARCHAR2(200),
  state VARCHAR2(20),
  zip VARCHAR2(20),
  country VARCHAR2(30)
);
/
CREATE TYPE lineitem_typ AS OBJECT(
  product_id VARCHAR2(10),
  product_name VARCHAR2(30),
  product_quantity NUMBER(30),
  product_price VARCHAR2(20)
);
/
CREATE TYPE lineitem_list AS TABLE OF lineitem_typ;
/
CREATE TYPE po_typ AS OBJECT(
 po_id raw(16),
 shipping_addr address_typ,
 billing_addr address_typ,
 lineitems lineitem_list
);
/

INSERT INTO poid VALUES('16CDBA33D78540C88436709C66BEDDB7');
COMMIT;

-------------------------------------------------------------------------------
--  Procedure to insert XML data into PO tables
-------------------------------------------------------------------------------
create or replace procedure insertPO(doc in XMLTYPE) as
  v_id RAW(16);
  v_temp XMLTYPE;
  v_itemid varchar2(100);
  v_num number;
  i number;
BEGIN
   ----------------------------------------------------------------------------
   -- Insert into PO table
   ----------------------------------------------------------------------------
   SELECT current_id into v_id from poid  where rownum=1;
   INSERT INTO po 
     values (v_id,
        extract(doc,'/PURCHASEORDER/SHIPPING_ADDR/CUSTOMER_NAME/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/SHIPPING_ADDR/STREET/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/SHIPPING_ADDR/CITY/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/SHIPPING_ADDR/STATE/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/SHIPPING_ADDR/ZIP/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/SHIPPING_ADDR/COUNTRY/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/BILLING_ADDR/CUSTOMER_NAME/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/BILLING_ADDR/STREET/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/BILLING_ADDR/CITY/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/BILLING_ADDR/STATE/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/BILLING_ADDR/ZIP/text()').getStringVal(),
        extract(doc,'/PURCHASEORDER/BILLING_ADDR/COUNTRY/text()').getStringVal());
   ----------------------------------------------------------------------------
   -- Insert into LineItem and LineItems table
   ----------------------------------------------------------------------------
   i := 1;
   LOOP
    v_temp := doc.extract('/PURCHASEORDER/LINEITEMS/LINEITEM['||i||']');
    EXIT WHEN v_temp IS NULL;
    v_itemid := v_temp.extract('/LINEITEM/PRODUCT_ID/text()').getStringVal();
    select count(1) into v_num from lineitem 
     where product_id = v_itemid;
     if v_num = 0 then
     insert into lineitem
       values(v_temp.extract('/LINEITEM/PRODUCT_ID/text()').getStringVal(), 
            v_temp.extract('/LINEITEM/PRODUCT_NAME/text()').getStringVal(),
            v_temp.extract('/LINEITEM/PRODUCT_QUANTITY/text()').getStringVal(),
            v_temp.extract('/LINEITEM/PRODUCT_PRICE/text()').getStringVal());
      end if;
      insert into lineitems values(v_id, v_itemid);
     i := i +1;
    END LOOP;
END;
/
show errors;

-------------------------------------------------------------------------------
--  Create XMLType Object View
-------------------------------------------------------------------------------
CREATE OR REPLACE VIEW purchaseorder_xmltype OF XMLTYPE
WITH OBJECT OID (EXTRACT(sys_nc_rowinfo$,'PURCHASEORDER/PO_ID').getnumberval()) AS
  select sys_xmlgen(po_typ(po_id, address_typ(shipping_csname,
                              shipping_street,
                              shipping_city,
                              shipping_state,
                              shipping_zip,
                              shipping_country), 
                  address_typ(billing_csname,
                              billing_street,
                              billing_city,
                              billing_state,
                              billing_zip,
                              billing_country),                  
                 cast(multiset(
                               select lineitem.product_id, product_name, 
                               product_quantity, product_price                              
                               from lineitem, lineitems
                               where lineitems.po_id=po.po_id and
                                     lineitem.product_id=lineitems.product_id
                               ) as lineitem_list) 
                      ))
     from po;      
/
-------------------------------------------------------------------------------
--  Instead of Trigger for XMLType View
-------------------------------------------------------------------------------
DROP TRIGGER purchaseordert_insert_trigger;
CREATE TRIGGER purchaseordert_insert_trigger
INSTEAD OF INSERT ON purchaseorder_xmltype
FOR EACH ROW 
DECLARE
BEGIN
   insertPO(:new.SYS_NC_ROWINFO$);
END;
/
-------------------------------------------------------------------------------
--  Insert into View
-------------------------------------------------------------------------------
SET serverouput ON

DECLARE
   rows NUMBER;
   v_xml  VARCHAR2(4000):='<PURCHASEORDER>
 <SHIPPING_ADDR>
    <COUNTRY>US</COUNTRY>
    <CUSTOMER_NAME>Alice Smith</CUSTOMER_NAME>
    <STREET>123 Maple Street</STREET>
    <CITY>Mill Valley</CITY>
    <STATE>CA</STATE>
    <ZIP>90952</ZIP>
  </SHIPPING_ADDR>
  <BILLING_ADDR>
    <COUNTRY>US</COUNTRY>
    <CUSTOMER_NAME>Robert Smith</CUSTOMER_NAME>
    <STREET>8 Oak Avenue</STREET>
    <CITY>Old Town</CITY>
    <STATE>PA</STATE>
    <ZIP>95819</ZIP>
  </BILLING_ADDR>
  <LINEITEMS>
    <LINEITEM>
      <PRODUCT_ID>872-AA</PRODUCT_ID>
      <PRODUCT_NAME>Lawnmower</PRODUCT_NAME>
      <PRODUCT_QUANTITY>1</PRODUCT_QUANTITY>
      <PRODUCT_PRICE>148.95</PRODUCT_PRICE>
    </LINEITEM>
    <LINEITEM>
      <PRODUCT_ID>926-AA</PRODUCT_ID>
      <PRODUCT_NAME>Baby Monitor</PRODUCT_NAME>
      <PRODUCT_QUANTITY>1</PRODUCT_QUANTITY>
      <PRODUCT_PRICE>39.98</PRODUCT_PRICE>
    </LINEITEM>
  </LINEITEMS>
</PURCHASEORDER>';

BEGIN
    insert into purchaseorder_xmltype values(xmltype(v_xml));
END;
/
set long 1000000
select * from purchaseorder_xmltype;