/**
 * File Name: XSDPrintProcess.java
 * Description:
 *   Print the XML document in SAX into output stream 
 * Note:
 *   - To be able to plug-in to XDK XML Pipeline processor, the class extends
 *     the oracle.xml.pipeline.controller.Process class 
 *   - Here are the parameters:
 *     Inputs    - "xmlsourcep"(sax)
 *     Params    - None
 *     Outputs   - "printxml" (stream), 
 *     Outparams - None
 * Modified (MM/DD/YYYY)
 *   jiwang  12/08/2003 - Create
 */
package oracle.xml.sample.pipeline;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import oracle.xml.parser.v2.XMLSAXSerializer;
import oracle.xml.pipeline.controller.Process;
import oracle.xml.pipeline.controller.Input;
import oracle.xml.pipeline.controller.Output;
import oracle.xml.pipeline.controller.PipelineException;
import org.xml.sax.ContentHandler;


public class SAXPrintProcess extends Process {
  StringWriter sw=new StringWriter();
  PrintWriter out= new PrintWriter(sw);

  public SAXPrintProcess() {
  }

  public void initialize() {
    /* Set the supported xml infoset types for the inputs and outputs */
    Input i = getInput("pxmlsource");
    i.supportType(SAXSOURCE);
    i.supportType(DOCUMENT);
    Output o = getOutput("printxml");
    o.supportType(STREAMRESULT);
    o.supportType(DOCUMENT);
  }

  public ContentHandler SAXContentHandler() throws PipelineException
  {
    printHandler = new XMLSAXSerializer(out); 

    return printHandler;
  } 

  public void execute() throws PipelineException
  {
    Object isPrint= getInParamValue("isPrint");    
    if(isPrint == null) isPrint="false";
    if(isPrint.equals("true")) {
      try {
        OutputStream outstream = getOutputStream("printxml");
        outstream.write(sw.toString().getBytes());
        out.flush();
      } catch(IOException ex) {
        ex.printStackTrace();
      }

    }
  } 

  XMLSAXSerializer printHandler;
}

