/**
 * File Name: XMLDiffProcess.java
 * Description:
 *   Create the pipeline process what can diff the DOM of two XML documents
 * Note:
 *   - To be able to plug-in to XDK XML Pipeline processor, the class extends
 *     the oracle.xml.pipeline.controller.Process class and create a wrapper 
 *     around the oracle.xml.differ.XMLDiff
 *   - Here are the parameters:
 *     Inputs    - "xmlsource1" (stream), "xmlsource1"(stream)
 *     Params    - none
 *     Outputs   - "xsl"(stream)
 *     Outparams - "isDiff" (boolean)
 * Modified (MM/DD/YYYY)
 *   jiwang  12/02/2003 - Create
 */
package oracle.xml.sample.pipeline;

import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.net.URL;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import oracle.xml.differ.XMLDiff;
import oracle.xml.parser.v2.DOMParser;
import oracle.xml.parser.v2.XMLDocument;
import oracle.xml.parser.v2.XMLParser;
import oracle.xml.pipeline.controller.Input;
import oracle.xml.pipeline.controller.Output;
import oracle.xml.pipeline.controller.PipelineException;
import oracle.xml.pipeline.controller.Process;
import org.xml.sax.InputSource;

public class XMLDiffProcess extends Process {
  public XMLDiffProcess() {
  }

  public void initialize() {
    // Initailized inputs
    Input input1 = getInput("xmlsource1");
    input1.supportType(DOMSOURCE);    
    input1.supportType(DOCUMENT);
    Input input2 = getInput("xmlsource2");
    input2.supportType(DOMSOURCE);    
    input2.supportType(DOCUMENT);
    // Initailized Output
    Output output = getOutput("xsl");
    output.supportType(STREAMRESULT);
    output.supportType(DOCUMENT);
  }

  public void execute() throws PipelineException
  {
    XMLDocument doc1=null, doc2=null;
    boolean res=false;
    DOMParser parser = new DOMParser();

    /* get input stream source */
    Source xmlSource1 = getInputSource("xmlsource1");
    Source xmlSource2 = getInputSource("xmlsource2");
    XMLDiff xmlDiff = new XMLDiff();
    if(xmlSource1 instanceof StreamSource) {
      StreamSource xmlstream = (StreamSource)xmlSource1;
      Object instream;
      try {
        if((instream = xmlstream.getInputStream()) != null) {
          parser.parse((InputStream)instream);
        } else {
          error(30005, FATAL_ERROR, new String[]{"xmlsource"});
        }
        doc1= parser.getDocument();
        //doc1.print(System.out); // Remove after debug
      } catch(Exception e) {
        e.printStackTrace();
      }
    } else {
      error(30002, FATAL_ERROR, new String[]{"StreamSource"});
    }
    if(xmlSource2 instanceof StreamSource) {
      StreamSource xmlstream = (StreamSource)xmlSource2;
      Object instream;
      try {
        if((instream = xmlstream.getInputStream()) != null) {
          parser.parse((InputStream)instream);
        } else {
          error(30005, FATAL_ERROR, new String[]{"xmlsource"});
        }
        doc2= parser.getDocument();
        //doc2.print(System.out); // Remove after debug
      } catch(Exception e) {
        e.printStackTrace();
      }
    } else {
      error(30002, FATAL_ERROR, new String[]{"StreamSource"});
    }
    if(doc1!=null && doc2 !=null) {
      xmlDiff.setInput1(doc1);
      xmlDiff.setInput2(doc2);
      res = xmlDiff.diff();
      if(res == true) setOutParam("isDiff","true");
      else setOutParam("isDiff","false");
       //System.out.println(res); // Remove after debugging
    } else setOutParam("isDiff","N/A");
    if(res==true){
      try {
        XMLDocument doc = xmlDiff.generateXSLDoc(); 
        OutputStream outstream = getOutputStream("xsl");
        doc.print(outstream);  
      } catch(Exception e) {
        e.printStackTrace();
      }
    }
    return;
  }
  public void endExecute() throws PipelineException
  {
    System.out.println("End of XMLDiffProcess!!");
  } 

}