/**
 * File Name: XSDConditionValProcess.java
 * Description:
 *   A switch whose behaviors are on the status of the XML Schema Validation
 * Note:
 *   - To be able to plug-in to XDK XML Pipeline processor, the class extends
 *     the oracle.xml.pipeline.controller.Process class 
 *   - Here are the parameters:
 *     Inputs    - "xml" (sax), "xmlsource1"()
 *     Params    - None
 *     Outputs   - "validatedxml" (sax, stream, dom), 
 *     Outparams - None
 * Modified (MM/DD/YYYY)
 *   jiwang  12/08/2003 - Clean up
 *   jiwang  12/03/2003 - Create
 */
package oracle.xml.sample.pipeline;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import oracle.xml.comp.CXMLContext;
import oracle.xml.comp.CXMLStream;
import oracle.xml.parser.schema.XSDConstantValues;
import oracle.xml.parser.v2.XMLDocument;
import oracle.xml.pipeline.controller.Input;
import oracle.xml.pipeline.controller.Output;
import oracle.xml.pipeline.controller.Process;
import oracle.xml.pipeline.controller.PipelineException;
import org.xml.sax.ContentHandler;

public class XSDConditionalValProcess extends Process {
  StringWriter sw=new StringWriter();
  PrintWriter out= new PrintWriter(sw);
  boolean isValid=true;

  public XSDConditionalValProcess() {
  }

  public void initialize() {
    /* Set the supported xml infoset types for the inputs and outputs */
    Input i = getInput("csax");
    i.supportType(SAXSOURCE);
    i.supportType(DOCUMENT);
    Output o = getOutput("xmlout");
    o.supportType(SAXRESULT);
    o.supportType(STREAMRESULT);
    o.supportType(DOCUMENT);
  }

  public ContentHandler SAXContentHandler() throws PipelineException
  {
    try {
      Output output = getOutput("xmlout");
      xsdHandler = new XSDValidationHandler(new PrintWriter(out), this);    
      xsdHandler.setXMLProperty(XSDConstantValues.VALIDATION_MODE, 
                                XSDConstantValues.STRICT_VALIDATION);
      ContentHandler hdlr = getSAXContentHandler(output);
      xsdHandler.addContentHandler(hdlr);
      Object schemaObj = getInParamValue("txmlschema");    
      Object obj = xsdHandler.setXMLProperty(XSDConstantValues.FIXED_SCHEMA,
                                             schemaObj);
      if(obj== null)
        throw new PipelineException("The XML Schema Object is not correct.");
    } catch(Exception ex) {
      ex.printStackTrace();
    }

    return xsdHandler;
  } 

  public void execute() throws PipelineException
  {
    //System.out.println("Start of ConditionalValProcess!!"); //Remove after debuging
    //super.execute();
    Object isCheck = getInParamValue("isCheck");  
    if(isCheck.equals("true")) {
      Object isDiff = getInParamValue("isXSDDiff");
      if(isDiff.equals("true")) {
        System.out.println("XSD document are not consistent.");
        System.out.println("Please upload the new XSD file.");
        if(isValid==true) {
          System.out.println("XML document accepted.");
          setOutParam("isValid","true");
        } else {
          System.out.println("XML document rejected.");
          setOutParam("isValid","false");
        }
      } else {
        System.out.println("XSD document are consistent.");
        System.out.println("XML document rejected.");
        setOutParam("isValid","false");
      }
    } else if(isCheck.equals("ignore")) {
      setOutParam("isValid","true");      
      System.out.println("XML document accepted.");
    } else if(isCheck.equals("false")) {
      setOutParam("isValid","false");      
      System.out.println("XML document rejected.");
    }

    //System.out.println(sw.toString());                      //Remove after debuging
    //System.out.println("End of XSD ConditionalValProcess!!"); //Remove after debuging
  } 

  XSDValidationHandler xsdHandler;
}

