/**
 * File Name: XSDSwitchProcess.java
 * Description:
 *   A switch whose behaviors depending on the status of the XML Schema validation
 * Note:
 *   - To be able to plug-in to XDK XML Pipeline processor, the class extends
 *     the oracle.xml.pipeline.controller.Process class 
 *   - Here are the parameters:
 *     Inputs    - "xml" (sax)
 *     Params    - "sschemaObj" (XMLSchema Object)
 *     Outputs   - "validatedxml" (sax), 
 *     Outparams - None
 * Modified (MM/DD/YYYY)
 *   jiwang  12/08/2003 - Clean up
 *   jiwang  12/03/2003 - Create
 */
package oracle.xml.sample.pipeline;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import oracle.xml.comp.CXMLContext;
import oracle.xml.comp.CXMLStream;
import oracle.xml.parser.schema.XSDConstantValues;
import oracle.xml.parser.v2.XMLDocument;
import oracle.xml.pipeline.controller.Input;
import oracle.xml.pipeline.controller.Output;
import oracle.xml.pipeline.controller.Process;
import oracle.xml.pipeline.controller.PipelineException;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;

public class XSDValProcess extends Process {
  StringWriter sw=new StringWriter();
  PrintWriter out= new PrintWriter(sw);
  boolean isCheckXSDDiff=false;
  boolean isValid=true;

  public XSDValProcess() {
  }

  public void initialize() {
    /* Set the supported xml infoset types for the inputs and outputs */
    Input i = getInput("xml");
    i.supportType(SAXSOURCE);
    i.supportType(DOCUMENT);
    Output o = getOutput("validatedxml");
    //o.supportType(DOMRESULT);
    o.supportType(SAXRESULT);
    o.supportType(STREAMRESULT);
    o.supportType(DOCUMENT);
  }

  public ContentHandler SAXContentHandler() throws PipelineException {
    try {
      Output output = getOutput("validatedxml");
      // if dependents are not available then return null
      ContentHandler hdlr = getSAXContentHandler(output);
      if (hdlr == null)
         return null;
      Object schemaObj = getInParamValue("sschemaObj");
      if (schemaObj == null)
         return null;
      exXsdHandler = new ExXSDHandler(new PrintWriter(out), this); 
      exXsdHandler.addContentHandler(hdlr);
      exXsdHandler.setXMLProperty(XSDConstantValues.VALIDATION_MODE, 
                                XSDConstantValues.STRICT_VALIDATION);    
      Object obj = exXsdHandler.setXMLProperty(XSDConstantValues.FIXED_SCHEMA,
                                             schemaObj);
      if(obj== null)
        throw new PipelineException("The XML Schema Object is not correct.");    

    } catch(Exception ex) {
      ex.printStackTrace();
    }

    return exXsdHandler;
  } 

  public void execute() throws PipelineException {
    //System.out.println("Start of XSD Switch!!");    // Remove after debuging
    super.execute();
    if(isValid==true) {
      //System.out.println("XML document accepted."); // Remove after debuging      
      setOutParam("isCheckXSDDiff","ignore");
    } else {
      //Set the output parameter
      if(isCheckXSDDiff == true) setOutParam("isCheckXSDDiff","true");
      else {
        setOutParam("isCheckXSDDiff","false");
        //System.out.println("XML document rejected."); // Remove after debuging 
      }
      System.out.println(sw.toString());              // Remove after debuging
    }
   // System.out.println("End of XSD Switch");          // Remove after debuging
  } 

  ExXSDHandler exXsdHandler;
}

