/* declare SAX callback functions */
sword startdocument(void *ctx);
sword enddocument(void *ctx);
sword startelement(void *ctx, const oratext *name,
const xmlnodes *attrs);
sword endelement(void *ctx, const oratext *name);
sword characters(void *ctx, const oratext *ch, size_t len);
sword whitespace(void *ctx, const oratext *ch, size_t len);
sword pi(void *ctx, const oratext *target,
const oratext *data);
sword notation(void *ctx, const oratext *name,
const oratext *publicId,
const oratext *systemId);
sword entity(void *ctx, const oratext *name,
const oratext *publidId,
const oratext *systemId,
const oratext *notationName);
/* declare SAX callback context */
typedef struct saxcontext {
uword depth; /* nested element level, for indenting */
} sax_context;
/* declare SAX callback structure */
xmlsaxcb sax_callback = {
startdocument, enddocument, startelement, endelement,
characters, whitespace, pi, notation, entity
};
/* declare SAX context and initialize */
sax_context saxctx = { 0 }; /* depth = 0 */
/* initialize parser specifying SAX callbacks */
xmlinit(&ecode, NULL, NULL, NULL, NULL,
&sax_callback, (void *) &saxctx, NULL, NULL);
/* ----- SAX CALLBACKS ----- */
sword startdocument(void *context) {
puts("StartDocument");
return 0; /* success */
}
sword enddocument(void *context) {
puts("EndDocument");
return 0; /* success */
}
sword startelement(void *context, const oratext *name,
const xmlnodes *attrs) {
sax_context *saxctx = (sax_context *) context;
indent(saxctx->depth);
printf("<%s", name);
if (attrs) {
for (i = 0; i < numAttributes(attrs); i++) {
attr = getAttributeIndex(attrs, i);
printf(" %s=\"%s\"", getAttrName(attr), getAttrValue(attr));
}
}
puts(">");
saxctx->depth++;
return 0; /* success */
}
sword endelement(void *context, const oratext *name) {
sax_context *saxctx = (sax_context *) context;
indent(--saxctx->depth);
printf("</%s>\n", name);
return 0; /* success */
}
sword characters(void *context, const oratext *ch, size_t len) {
sax_context *saxctx = (sax_context *) context;
indent(saxctx->depth);
putchar('"');
print_string((oratext *) ch, (sword) len);
puts("\"");
return 0; /* success */
}
sword whitespace(void *context, const oratext *ch, size_t len)
{
sax_context *saxctx = (sax_context *) context;
indent(saxctx->depth);
putchar('\'');
print_string((oratext *) ch, (sword) len);
puts("'");
return 0; /* success */
}
sword pi(void *context, const oratext *target,
const oratext *data) {
sax_context *saxctx = (sax_context *) context;
indent(saxctx->depth);
fputs("PI", stdout);
if (target)
printf(" target=\"%s\"", target);
if (data)
printf (" data=\"%s\"", data);
putchar('\n');
return 0; /* success */
}
sword notation(void *context, const oratext *name,
const oratext *publicId,
const oratext *systemId) {
sax_context *saxctx = (sax_context *) context;
indent(saxctx->depth);
printf("NOTATION '%s'", name);
if (publicId)
printf (" PUB:%s", publicId);
if (systemId)
printf(" SYS:%s", systemId);
putchar('\n');
return 0; /* success */
}
sword entity(void *context, const oratext *name,
const oratext *publidId,
const oratext *systemId,
const oratext *notationName) {
sax_context *saxctx = (sax_context *) context;
indent(saxctx->depth);
printf("ENTITY '%s'", name);
if (publidId)
printf(" PUB:%s", publidId);
if (systemId)
printf(" SYS:%s", systemId);
if (notationName)
printf(" NAME:%s", notationName);
putchar('\n');
return 0; /* success */
}