#include "xmlupdate.h"

/*---------------------------Main function -----------------------------*/
int main(argc, argv)
int argc;
char *argv[];
{
  /* OCI Handles and Variables */
 OCIEnv        *envhp = (OCIEnv *) 0;
 OCIServer     *srvhp = (OCIServer *) 0;
 OCISvcCtx     *svchp = (OCISvcCtx *) 0;
 OCIError      *errhp = (OCIError *) 0;
 OCISession    *authp = (OCISession *) 0;
 OCIStmt       *stmthp = (OCIStmt *) 0;

  /* database login information */
 text          *user=(text *)"SH";
 text          *password=(text *)"SH";
 
  /* Misellaneous */
 boolean        tab_exists = TRUE;
 ub4            row = 1;

 if (argc < 3) {
   printf("\nUsage error: update \"<SQL/XML Query>\" \"<XPath>\" <value>\n\n");
   return OCI_ERROR;
 }

 /* Initialize the environment and allocate handles */
 if (init_env_handle(&envhp, &svchp, &errhp, &srvhp, &authp, &stmthp)) 
 {
   printf("FAILED: init_env_handle()!\n");
   return OCI_ERROR;
 }

 /* Log on to the server and begin a session */
 if (connect_server(srvhp, errhp, svchp, authp, user, password)) 
 {
   printf("FAILED: connect_server()!");
   cleanup(envhp);
   return OCI_ERROR;
 }

 /* Select an xmltype column by defining it to an xmltype instance,
       apply some OCIXMLType and unified DOM APIs */ 
 row = 1; 

 if (select_xml(envhp, svchp, errhp, stmthp, row, (text *)argv[1], 
                      (text *)argv[2], (text *)argv[3]))
 {
   printf("FAILED: select_xml()!\n");
   disconnect_server(envhp, svchp, errhp, authp, srvhp, stmthp, tab_exists);
   return OCI_ERROR;
 }

 /* Detach from a server and clean up the environment */
 disconnect_server(envhp, svchp, errhp, authp, srvhp, stmthp, tab_exists);

 return OCI_SUCCESS;
}


/*---------------------------Subfunctions -----------------------------*/

/*--------------------------------------------------------*/
/* Return corresponding messages in differrent cases */
/*--------------------------------------------------------*/
void checkerr(errhp, status)
OCIError *errhp;
sword     status;
{
  text  msgbuf[512];
  sb4   errcode = 0;

  memset((void *) msgbuf, (int)'\0', (size_t)512);

  switch (status)
  {
  case OCI_SUCCESS: break;
  case OCI_SUCCESS_WITH_INFO:
    printf("status = OCI_SUCCESS_WITH_INFO\n");
    OCIErrorGet((dvoid *) errhp, (ub4) 1, (text *) NULL, &errcode,
                       msgbuf, (ub4) sizeof(msgbuf), (ub4) OCI_HTYPE_ERROR);
    printf("ERROR CODE = %d\n", errcode);
    printf("%.*s\n", 512, msgbuf);
    if (errcode == 436 || errcode == 437 || errcode == 438 || errcode == 439)
      exit(1);
    break;
  case OCI_NEED_DATA:
    printf("status = OCI_NEED_DATA\n");
    break;
  case OCI_NO_DATA:
     printf("status = OCI_NO_DATA\n");
    break;
  case OCI_ERROR:
     printf("status = OCI_ERROR\n");
     OCIErrorGet((dvoid *) errhp, (ub4) 1, (text *) NULL, &errcode,
                       msgbuf, (ub4) sizeof(msgbuf), (ub4) OCI_HTYPE_ERROR);
     printf("ERROR CODE = %d\n", errcode);
     printf("%.*s\n", 512, msgbuf);
    if (errcode == 436 || errcode == 437 || errcode == 438 || errcode == 439)
      exit(1);
    break;
  case OCI_INVALID_HANDLE:
     printf("status = OCI_INVALID_HANDLE\n");
    break;
  case OCI_STILL_EXECUTING:
     printf("status = OCI_STILL_EXECUTE\n");
    break;
  case OCI_CONTINUE:
     printf("status = OCI_CONTINUE\n");
    break;
  default:
    break;
  }

  return;
}


/*--------------------------------------------------------*/
/* Free the envhp whenever there is an error  */
/*--------------------------------------------------------*/
void cleanup(envhp) 
OCIEnv *envhp;
{
 if (envhp) {
    OCIHandleFree((dvoid *)envhp, OCI_HTYPE_ENV);
 }

 return;
}


/*--------------------------------------------------------*/
/* attach to server, set attributes, and begin session */
/*--------------------------------------------------------*/
sb4 connect_server(srvhp, errhp, svchp, authp, user, password, conn)
OCIServer *srvhp;
OCIError *errhp;
OCISvcCtx *svchp;
OCISession *authp;
text *user;
text *password;
text *conn;
{
 sword status = 0;
 /* attach to server */
 if (status = OCIServerAttach(srvhp, errhp, (text *) "", (sb4) strlen(""), (ub4) OCI_DEFAULT)) 
 {
    printf("FAILED: OCIServerAttach() on srvhp\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }
 /* set server attribute to service context */
 if (status = OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX, 
                         (dvoid *) srvhp, (ub4) 0, OCI_ATTR_SERVER, 
                         (OCIError *) errhp)) 
 {
    printf("FAILED: OCIAttrSet() on svchp\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }
 /* set user attribute to session */
 if (status = OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, 
                         (dvoid *) user, (ub4) strlen((char *)user), 
                         (ub4) OCI_ATTR_USERNAME, (OCIError *) errhp)) 
 { 
    printf("FAILED: OCIAttrSet() on authp for user\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }
 /* set password attribute to session */
 if (status = OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION, 
                         (dvoid *) password, (ub4) strlen((char *)password), 
                         (ub4) OCI_ATTR_PASSWORD, (OCIError *) errhp)) 
 { 
    printf("FAILED: OCIAttrSet() on authp for password\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }
 /* Begin a session  */
 if (status = OCISessionBegin((OCISvcCtx *) svchp, (OCIError *) errhp, 
                              (OCISession *) authp, (ub4) OCI_CRED_RDBMS, 
                              (ub4) OCI_DEFAULT)) 
 {
    printf("FAILED: OCISessionBegin()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 } 
 /* set session attribute to service context */
 if (status = OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX, 
                         (dvoid *) authp, (ub4) 0, (ub4) OCI_ATTR_SESSION, 
                         (OCIError *) errhp)) 
 {
    printf("FAILED: OCIAttrSet() on svchp\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 return OCI_SUCCESS;
}


/*--------------------------------------------------------*/
/* End the session, detach server and free handles. */
/*--------------------------------------------------------*/
void disconnect_server(envhp, svchp, errhp, authp, srvhp, stmthp, tab_exists)
OCIEnv *envhp;
OCISvcCtx *svchp;
OCIError *errhp;
OCISession *authp;
OCIServer *srvhp;
OCIStmt *stmthp;
boolean tab_exists;
{
 sword status = 0;

  printf("\n\nLogged off and detached from server.\n");

   /* End a session */
 if (status = OCISessionEnd((OCISvcCtx *)svchp, (OCIError *)errhp, 
                           (OCISession *)authp, (ub4) OCI_DEFAULT)) { 
   checkerr(errhp, status);
   cleanup(envhp);
   return; 
 }

   /* Detach from the server */
 if (status = OCIServerDetach((OCIServer *)srvhp, (OCIError *)errhp, 
                             (ub4)OCI_DEFAULT)) {
   checkerr(errhp, status);
   cleanup(envhp);
   return;
 } 

   /* Free the handles */
 if (stmthp) {
    OCIHandleFree((dvoid *)stmthp, (ub4) OCI_HTYPE_STMT); 
 }
 if (authp) {
    OCIHandleFree((dvoid *)authp, (ub4) OCI_HTYPE_SESSION); 
 }
 if (svchp) {
    OCIHandleFree((dvoid *)svchp, (ub4) OCI_HTYPE_SVCCTX); 
 }
 if (srvhp) {
    OCIHandleFree((dvoid *)srvhp, (ub4) OCI_HTYPE_SERVER); 
 }
 if (errhp) {
    OCIHandleFree((dvoid *)errhp, (ub4) OCI_HTYPE_ERROR); 
 }
 if (envhp) {
    OCIHandleFree((dvoid *)envhp, (ub4) OCI_HTYPE_ENV); 
 }

  return;
}


/*--------------------------------------------------------*/
/* Initialize the environment and allocate handles */
/*--------------------------------------------------------*/
sb4 init_env_handle(envhp, svchp, errhp, srvhp, authp, stmthp)
OCIEnv **envhp;
OCISvcCtx **svchp;
OCIError **errhp;
OCIServer **srvhp;
OCISession **authp;
OCIStmt **stmthp;
{
 sword status = 0;

 /* Environment initialization and creation */
 if (OCIEnvCreate((OCIEnv **) envhp, (ub4) OCI_OBJECT, (dvoid *) 0, 
                  (dvoid * (*)(dvoid *,size_t)) 0, 
                  (dvoid * (*)(dvoid *, dvoid *, size_t)) 0, 
                  (void (*)(dvoid *, dvoid *)) 0, (size_t) 0, (dvoid **) 0)) 
 {
    printf("FAILED: OCIEnvCreate()\n");
   return OCI_ERROR;
 }
 
 /* allocate error handle */
 if (OCIHandleAlloc((dvoid *) *envhp, (dvoid **) errhp, 
                (ub4) OCI_HTYPE_ERROR, (size_t) 0, (dvoid **) 0)) 
 {
    printf("FAILED: OCIHandleAlloc() on errhp\n");
   return OCI_ERROR;
 }

 /* allocate server handle */
 if (status = OCIHandleAlloc((dvoid *) *envhp, (dvoid **) srvhp, 
                        (ub4) OCI_HTYPE_SERVER, (size_t) 0, (dvoid **) 0)) 
 {
    printf("FAILED: OCIHandleAlloc() on srvhp\n");
   checkerr(*errhp, status);
   return OCI_ERROR;
 }
 
 /* allocate service context handle */
 if (status = OCIHandleAlloc((dvoid *) *envhp, (dvoid **) svchp, 
                        (ub4) OCI_HTYPE_SVCCTX, (size_t) 0, (dvoid **) 0)) 
 {
    printf("FAILED: OCIHandleAlloc() on svchp\n");
   checkerr(*errhp, status);
   return OCI_ERROR;
 }

 /* allocate session handle */
 if (status = OCIHandleAlloc((dvoid *) *envhp, (dvoid **) authp, 
                      (ub4) OCI_HTYPE_SESSION, (size_t) 0, (dvoid **) 0)) 
 {
    printf("FAILED: OCIHandleAlloc() on authp\n");
   checkerr(*errhp, status);
   return OCI_ERROR;
 }

 /* Allocate statement handle */
 if (status = OCIHandleAlloc((dvoid *) *envhp, (dvoid **) stmthp, 
                       (ub4) OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0)) 
 {
    printf("FAILED: OCIHandleAlloc() on stmthp\n");
   checkerr(*errhp, status);
   return OCI_ERROR;
 }

 return OCI_SUCCESS;
}


/*--------------------------------------------------------*/
/* select an xml column from the table by defining it to an xmltype 
   instance, print it out using OCIPStream APIs, and then apply
   some OCIXMLType and unified DOM APIs on it */
/*--------------------------------------------------------*/
sb4 select_xml(envhp, svchp, errhp, stmthp, row, selstmt, xpathexpr, value)
OCIEnv *envhp;
OCISvcCtx *svchp;
OCIError *errhp;
OCIStmt *stmthp;
ub4 row;
text *selstmt;
text *xpathexpr;
text *value;
{
 ub4 colc = row;
 ub4 xmlsize = 0;
 sword status = 0;
 OCIDefine *defnp = (OCIDefine *) 0;
 OCIBind *bndhp = (OCIBind *) 0;
 OCIXMLType *xml = (OCIXMLType *) 0;
 OCIType *xmltdo = (OCIType *) 0;
 OCIInd ind = OCI_IND_NULL;
 OCIInd *xml_ind = &ind;

 printf("\n=> Selecting row %d by defining to an XMLType instance......\n", row);


  /* get the tdo for the xmltype */
 if(status = OCITypeByName(envhp, errhp, svchp, (const text *) SCHEMA,
                   (ub4) strlen((char *)SCHEMA), (const text *) TYPE,
                   (ub4) strlen((char *)TYPE), (CONST text *) 0,
                   (ub4) 0, OCI_DURATION_SESSION, OCI_TYPEGET_HEADER,
                   (OCIType **) &xmltdo)) {
   printf("FAILED: OCITypeByName()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 if(!xmltdo)
 {
   printf("NULL tdo returned\n");
   return OCI_ERROR;
 }

  /* create an new xmltype instance */
 if(status = OCIObjectNew((OCIEnv *)envhp, (OCIError *)errhp, (CONST OCISvcCtx *)svchp,
                     (OCITypeCode) OCI_TYPECODE_OPAQUE, (OCIType *) xmltdo,
                     (dvoid *) 0, (OCIDuration) OCI_DURATION_SESSION, FALSE,
                     (dvoid **) &xml)) {
   printf("FAILED: OCIObjectNew()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 if(status = OCIStmtPrepare(stmthp, errhp, (OraText *)selstmt,
                    (ub4)strlen((char *)selstmt),
                    (ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT)) {
   printf("FAILED: OCIStmtPrepare()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

  /* define it to an xmltype instance */
 if(status = OCIDefineByPos(stmthp, &defnp, errhp, (ub4) 1, (dvoid *) 0,
                   (sb4) 0, SQLT_NTY, (dvoid *) 0, (ub2 *)0,
                   (ub2 *)0, (ub4) OCI_DEFAULT)) {
   printf("FAILED: OCIDefineByPos()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 if(status = OCIDefineObject(defnp, errhp, (OCIType *) xmltdo,
                            (dvoid **) &xml,
                            &xmlsize, (dvoid **) &xml_ind, (ub4 *) 0)) {
   printf("FAILED: OCIDefineObject()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 if(status = OCIStmtExecute(svchp, stmthp, errhp, (ub4) 1, (ub4) 0,
                 (CONST OCISnapshot*) 0, (OCISnapshot*) 0, (ub4) OCI_DEFAULT)) {
   printf("FAILED: OCIStmtExecute()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 /* test some xml APIs */
 printf("\n\n=> Test some OCI XML and DOM APIs ......\n\n");

 if(do_xml(envhp, svchp, errhp, xml, xpathexpr, value)) {
   printf("FAILED: do_xml()\n");
   return OCI_ERROR;
 }

  /* free instances */
 if(status = OCIObjectFree(envhp, errhp, (dvoid *) xml, (ub2) OCI_OBJECTFREE_FORCE)) {
   printf("FAILED: OCIObjectFree()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 return OCI_SUCCESS;
}


/*--------------------------------------------------------*/
/* Initialize XML context and call DOM APIs */
/*--------------------------------------------------------*/
sb4 do_xml(envhp, svchp, errhp, xml, xpathexpr, value)
OCIEnv *envhp;
OCISvcCtx *svchp;
OCIError *errhp;
OCIXMLType *xml;
text *xpathexpr;
text *value;
{
 boolean result = 0;
 OCIXMLType *newxml = (OCIXMLType *) 0;
 OCIXMLType *xmltran = (OCIXMLType *) 0;
 sword  status = 0;
 /* For XML DOM APIs */
 OCIStmt *stmthp = (OCIStmt *)0;
 OCIDuration dur = OCI_DURATION_SESSION; 
 struct xmlctx *xctx = (xmlctx *)0;
 xmldocnode *doc = (xmldocnode *)0;
 ocixmldbparam params[1];
   

 /* Get an XML context */
  params[0].name_ocixmldbparam = XCTXINIT_OCIDUR;
  params[0].value_ocixmldbparam = &dur;
  xctx = OCIXmlDbInitXmlCtx(envhp, svchp, errhp, params, 1);

   /* cast the xmltype value to an xmldomctx */
  doc = (xmldocnode *) xml;

   /* test xml unified C APIs */
  if(update_xml((xmlctx *)xctx, doc, xpathexpr, value)) {
    printf("FAILED: do_xml()\n");
    return OCI_ERROR;
  }

  /* free the statement handle */
 if (stmthp) {
   OCIHandleFree((dvoid *)stmthp, OCI_HTYPE_STMT);
 }

  /* free the xmlctx */
  OCIXmlDbFreeXmlCtx(xctx);

   /* free xml instances using OCIObjectFree */
 if(newxml &&
    (status = OCIObjectFree(envhp, errhp, (dvoid *) newxml, (ub2) OCI_OBJECTFREE_FORCE))) {
   printf("FAILED: OCIObjectFree()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 return OCI_SUCCESS;
}

/*--------------------------------------------------------*/
/* Test XML unified C APIs which are equivalent to OCIXMLType* APIs in 9i */
/*--------------------------------------------------------*/
sword update_xml(xctx, doc, xpathexpr, value)
xmlctx *xctx;
xmldocnode *doc;
OraText *xpathexpr;
OraText *value;
{
 xpctx *xpathctx = (xpctx *) 0;
 OraText *baseuri = (OraText *) 0;
 ub2 ctxpos = 1;
 ub2 ctxsize = 1;
 xpexpr *exprtree = (xpexpr *) 0;
 xpobj *xpathobj = (xpobj *) 0;
 xmlerr xerr = (xmlerr)0; 
 boolean nodes_exists = 0;
 xmlnode *foo, *foobar; 
    
  /* print out the xml doc before applying XML APIs on it */
 printf("\nThe xml instance before applying xml unified APIs on it:\n");
 XmlSaveDom(xctx, &xerr, (xmlnode *)doc, "stdio", stdout, NULL);

  /* create an XPath context */ 
 xpathctx = XmlXPathCreateCtx(xctx, baseuri, (xmlnode *)doc, ctxpos, ctxsize);

  /* parse XPath expression */
 exprtree = XmlXPathParse((xpctx *)xpathctx, (OraText *)xpathexpr, (xmlerr *)&xerr);

  /* evaluate XPath expression */
 xpathobj = XmlXPathEval((xpctx *)xpathctx, (xpexpr *)exprtree, (xmlerr *)&xerr);

 foobar = XmlXPathGetObjectNSetNode(xpathobj, 0);

 foo = XmlDomGetFirstChild(xctx, foobar);

 XmlDomSetNodeValue(xctx, foo, (oratext *)value);

 printf("\nThe xml instance after applying xml unified APIs on it:\n");
 //XmlSaveDom(xctx, &xerr, (xmlnode *)doc, "stdio", stdout, NULL);

 return OCI_SUCCESS;
}

XML_ERRMSG_F(tkpgerr, ectx, msg, err)
{
  if (err)
  {
    printf("Error: %d\n", (int)err);
    printf("%s\n", msg);
  }
}
