/*

   NAME
     xsbtransform 

   DESCRIPTION
     Reads input XML mapping file, accepts input XML,
     and deposits output into possible output file.
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdlib.h>
#include <string.h>

#ifndef XML_ORACLE
#include <xml.h>
#endif

/* Obtains XSL compiled code file name */
oratext *getXSBFileName (xmlctx *xctx, xmldocnode *doc, oratext *xslmapfile)
{
   oratext       *p, *filename;
   static oratext input[100];
 
   xmlpinode     *stpinode, *pinode; 
   xmlnode       *stnode; 

   filename = (oratext *)malloc(200);

   /* Get first PI */
   pinode = (xmlpinode *)XmlDomGetFirstChild(xctx, (xmlnode *)doc);
   /* Get stylesheet PI */
   stpinode = (xmlpinode *)XmlDomGetNextSibling(xctx, (xmlnode *)pinode);
   /* Get href node as last child */
   stnode = XmlDomGetLastChild(xctx, (xmlnode *)stpinode);
   /* Get attribute value which is the filename */
   filename = XmlDomGetAttrValue(xctx, (xmlattrnode *)stnode);
   
   p = (oratext*)strtok((char*)filename, ".");
   strcpy((char*)input, (char*)p);
   strcat((char*)input, ".xsb"); 

   return input;
}



/* ======================================================================
                                 main
  ---------------------------------------------------------------------*/
int main (sword argc, char *argv[])
{
    xmlctx       *xctx = NULL;
    xmlxvm       *vm   = NULL;
    xmlerr        err;
    oratext      *xmlFile, *outFile, *xslmapFile, *codeFile;
    xmldocnode   *doc, *odoc; 
    xmlfragnode  *root;

    if (argc < 3) {
        printf("Usage is: xsbtransform <xmlfile> <outputfile> <xslmap file>\n");
        return 0;  
    }

    xmlFile = (oratext *)argv[1];
    outFile = (oratext *)argv[2];
    xslmapFile = (oratext *)argv[3]; 

    /* Get the xsl compiled code */

    /* Creates the context */
    xctx = XmlCreate(&err, (oratext *) "sample",
             "data_encoding", "US-ASCII", NULL);
    if (!xctx) {
        printf("Failed to create xctx: %d\n", err);     
        goto terminate;  
    }
if (!(doc=XmlLoadDom(xctx, &err, xmlFile, NULL)))
   {
       printf("Parse failed, code %d\n", (int) err);
   }
    codeFile = getXSBFileName(xctx, doc, xslmapFile);

    /* Creates the VM */
    vm = XmlXvmCreate (xctx, NULL);
    if (!vm) {
        printf("Failed to create VM");       
        goto terminate;
    }

    /* Create the output document node */
    odoc = XmlCreateDocument(xctx, NULL, NULL, NULL, &err);
    err = XmlXvmSetOutputDom(vm, odoc);

    err = XmlXvmSetBytecodeFile(vm, codeFile);
    if (err != XMLERR_OK) {
        printf("Failed to set the bytecode: %u\n", err);
        goto terminate;
    }    
 
    err = XmlXvmTransformDom (vm, doc);
    if (err != XMLERR_OK) {
        printf("Failed to transform: %u\n", err);
        goto terminate;
    }

    /* Get DOM of transformed xml file and write into output file */
    root = XmlXvmGetOutputDom(vm);
    XmlSaveDom(xctx, &err, root, "outFile", NULL);

    terminate:
    /* clean the memory */
    if (vm)
        XmlXvmDestroy(vm);
    if (xctx)
        XmlDestroy(xctx);

    return 0;
}
