#include "sample.hpp"

/*---------------------------Main function -----------------------------*/

int main( int argc, char* argv[])
{
  /* database login information */
  text          *user=(text *)"SCOTT";
  text          *password=(text *)"TIGER";

  /* OCI environment variables */
  ociapi* ociobjp;

  /* Misellaneous */
  boolean        tab_exists = TRUE;
  ub4            row = 1;

  if (argc < 2) {
    printf( "\nUsage error: select <SQL-XML Query> <XPath>\n\n");
    return OCI_ERROR;
  }

  ociobjp = new ociapi();

  /* Initialize the environment and allocate handles */
  if (ociobjp->init_env_handle())
  {
    printf("FAILED: init_env_handle()!\n");
    delete ociobjp;
    return OCI_ERROR;
  }

  /* Log on to the server and begin a session */
  if (ociobjp->connect_server( user, password))
  {
    printf("FAILED: connect_server()!\n");

    delete ociobjp;
    return OCI_ERROR;
  }

  /* Select an xmltype column by defining it to an xmltype instance,
     apply some OCIXMLType and unified DOM APIs */

  if (ociobjp->select_from_doc( row, (text *)argv[1], (text *)argv[2]))
  {
    printf("FAILED: select_from_doc!\n");
    ociobjp->disconnect_server( tab_exists);
    delete ociobjp;
    return OCI_ERROR;
  }

  /* Detach from a server and clean up the environment */
  ociobjp->disconnect_server( tab_exists);
  delete ociobjp;

  return OCI_SUCCESS;
}

ociapi::ociapi()
{
  envhp = (OCIEnv *) 0;
  srvhp = (OCIServer *) 0;
  svchp = (OCISvcCtx *) 0;
  errhp = (OCIError *) 0;
  authp = (OCISession *) 0;
  stmthp = (OCIStmt *) 0;
}

ociapi::~ociapi()
{
  cleanup();
}

/*--------------------------------------------------------*/
/* Initialize the environment and allocate handles */
/*--------------------------------------------------------*/
sb4 ociapi::init_env_handle()
{
  sword status = 0;

  /* Environment initialization and creation */
  if (OCIEnvCreate((OCIEnv **) &envhp, (ub4) OCI_OBJECT, (dvoid *) 0,
		   (dvoid * (*)(dvoid *,size_t)) 0,
		   (dvoid * (*)(dvoid *, dvoid *, size_t)) 0,
		   (void (*)(dvoid *, dvoid *)) 0, (size_t) 0, (dvoid **) 0))
  {
    printf("FAILED: OCIEnvCreate()\n");
    return OCI_ERROR;
  }

  /* allocate error handle */
  if (OCIHandleAlloc( (dvoid *)envhp, (dvoid **) errhp,
		      (ub4) OCI_HTYPE_ERROR, (size_t) 0, (dvoid **) 0))
  {
    printf("FAILED: OCIHandleAlloc() on errhp\n");
    return OCI_ERROR;
  }

  /* allocate server handle */
  if (status = OCIHandleAlloc( (dvoid*)envhp, (dvoid **) &srvhp,
			      (ub4) OCI_HTYPE_SERVER, (size_t) 0,
			      (dvoid **) 0))
  {
    printf("FAILED: OCIHandleAlloc() on srvhp\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  /* allocate service context handle */
  if (status = OCIHandleAlloc(( dvoid*)envhp, (dvoid **) &svchp,
                        (ub4) OCI_HTYPE_SVCCTX, (size_t) 0, (dvoid **) 0))
  {
    printf("FAILED: OCIHandleAlloc() on svchp\n");

    checkerr( errhp, status);
    return OCI_ERROR;
  }

  /* allocate session handle */
  if (status = OCIHandleAlloc( (dvoid*)envhp, (dvoid **) &authp,
			      (ub4) OCI_HTYPE_SESSION, (size_t) 0,
			      (dvoid **) 0))
  {
    printf("FAILED: OCIHandleAlloc() on authp\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  /* Allocate statement handle */
  if (status = OCIHandleAlloc( (dvoid*)envhp, (dvoid **) stmthp,
			      (ub4) OCI_HTYPE_STMT, (size_t) 0, (dvoid **) 0))
  {
    printf("FAILED: OCIHandleAlloc() on stmthp\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  return OCI_SUCCESS;
}

sb4 ociapi::connect_server( text* user, text* password)
{
  sword status = 0;

  /* attach to server */
  if (status = OCIServerAttach((OCIServer *) srvhp, (OCIError *) errhp,
			       (text *) "", (sb4) strlen(""),
			       (ub4) OCI_DEFAULT))
  {
    printf("FAILED: OCIServerAttach() on srvhp\n");
    checkerr(errhp, status);
    return OCI_ERROR;
  }

  /* set server attribute to service context */
  if (status = OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX,
			  (dvoid *) srvhp, (ub4) 0, (ub4) OCI_ATTR_SERVER,
			  (OCIError *) errhp))
  {
    printf("FAILED: OCIAttrSet() on svchp\n");
    checkerr(errhp, status);
    return OCI_ERROR;
  }

  /* set user attribute to session */
  if (status = OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION,
			  (dvoid *) user, (ub4) strlen((char *)user),
			  (ub4) OCI_ATTR_USERNAME, (OCIError *) errhp))
  {
    printf("FAILED: OCIAttrSet() on authp for user\n");
    checkerr(errhp, status);
    return OCI_ERROR;
  }

  /* set password attribute to session */
  if (status = OCIAttrSet((dvoid *) authp, (ub4) OCI_HTYPE_SESSION,
			  (dvoid *) password, (ub4) strlen((char *)password),
			  (ub4) OCI_ATTR_PASSWORD, (OCIError *) errhp))
  {
    printf("FAILED: OCIAttrSet() on authp for password\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  /* Begin a session  */
  if (status = OCISessionBegin((OCISvcCtx *) svchp, (OCIError *) errhp,
			       (OCISession *) authp, (ub4) OCI_CRED_RDBMS,
			       (ub4) OCI_DEFAULT))
  {
    printf("FAILED: OCISessionBegin()\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  /* set session attribute to service context */
  if (status = OCIAttrSet((dvoid *) svchp, (ub4) OCI_HTYPE_SVCCTX,
			  (dvoid *) authp, (ub4) 0, (ub4) OCI_ATTR_SESSION,
			  (OCIError *) errhp))
  {
    printf("FAILED: OCIAttrSet() on svchp\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  return OCI_SUCCESS;
}

/*--------------------------------------------------------*/
/* End the session, detach server and free handles. */
/*--------------------------------------------------------*/
void ociapi::disconnect_server( boolean tab_exists)
{
  sword status = 0;

  printf("\n\nLogged off and detached from server.\n");

  /* End a session */
  if (status = OCISessionEnd((OCISvcCtx *)svchp, (OCIError *)errhp,
			     (OCISession *)authp, (ub4) OCI_DEFAULT)) {
    checkerr( errhp, status);
    cleanup();
    return;
  }

  /* Detach from the server */
  if (status = OCIServerDetach((OCIServer *)srvhp, (OCIError *)errhp,
			       (ub4)OCI_DEFAULT)) {
    checkerr( errhp, status);
    cleanup();
    return;
  }

  /* Free the handles */
  if (stmthp) {
    OCIHandleFree((dvoid *)stmthp, (ub4) OCI_HTYPE_STMT);
  }
  if (authp) {
    OCIHandleFree((dvoid *)authp, (ub4) OCI_HTYPE_SESSION);
  }
  if (svchp) {
    OCIHandleFree((dvoid *)svchp, (ub4) OCI_HTYPE_SVCCTX);
  }
  if (srvhp) {
    OCIHandleFree((dvoid *)srvhp, (ub4) OCI_HTYPE_SERVER);
  }
  if (errhp) {
    OCIHandleFree((dvoid *)errhp, (ub4) OCI_HTYPE_ERROR);
  }
  if (envhp) {
    OCIHandleFree((dvoid *)envhp, (ub4) OCI_HTYPE_ENV);
  }
}

void ociapi::cleanup()
{
  if (envhp) {
    OCIHandleFree((dvoid *)envhp, OCI_HTYPE_ENV);
  }
}

/*--------------------------------------------------------*/
/* select an xml column from the table by defining it to an xmltype
   instance, print it out using OCIPStream APIs, and then apply
   some OCIXMLType and unified DOM APIs on it */
/*--------------------------------------------------------*/
sb4 ociapi::select_from_doc( ub4 row, text* select_stmt, text *xpathexpr)
{
  ub4 colc = row;
  ub4 xmlsize = 0;
  sword status = 0;
  OCIDefine *defnp = (OCIDefine *) 0;
  OCIBind *bndhp = (OCIBind *) 0;
  OCIXMLType *xml = (OCIXMLType *) 0;
  OCIType *xmltdo = (OCIType *) 0;
  OCIInd ind = OCI_IND_NULL;
  OCIInd *xml_ind = &ind;

  printf("\n=> Selecting row %d by an XMLType instance......\n", row);

  /* get the tdo for the xmltype */
  if (status = OCITypeByName(envhp, errhp, svchp, (const text *) SCHEMA,
			     (ub4) strlen((char *)SCHEMA), (const text *) TYPE,
			     (ub4) strlen((char *)TYPE), (CONST text *) 0,
			     (ub4) 0, OCI_DURATION_SESSION, OCI_TYPEGET_HEADER,
			     (OCIType **) &xmltdo))
  {
    printf("FAILED: OCITypeByName()\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  if (!xmltdo)
  {
    printf("NULL tdo returned\n");
    return OCI_ERROR;
  }

  /* create an new xmltype instance */
  if (status = OCIObjectNew( (OCIEnv*)envhp, (OCIError*)errhp,
			     (CONST OCISvcCtx *)svchp,
			     (OCITypeCode) OCI_TYPECODE_OPAQUE,
			     (OCIType *) xmltdo, (dvoid *) 0,
			     (OCIDuration) OCI_DURATION_SESSION, FALSE,
			     (dvoid **) &xml))
  {
    printf("FAILED: OCIObjectNew()\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  if(status = OCIStmtPrepare( stmthp, errhp, (OraText *)select_stmt,
			      (ub4)strlen((char*)select_stmt),
			      (ub4) OCI_NTV_SYNTAX, (ub4) OCI_DEFAULT))
  {
    printf("FAILED: OCIStmtPrepare()\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  /* define it to an xmltype instance */
  if(status = OCIDefineByPos( stmthp, &defnp, errhp, (ub4) 1, (dvoid *) 0,
			      (sb4) 0, SQLT_NTY, (dvoid *) 0, (ub2 *)0,
			      (ub2 *)0, (ub4) OCI_DEFAULT))
  {
    printf("FAILED: OCIDefineByPos()\n");
    checkerr( errhp, status);
    return OCI_ERROR;
  }

  if(status = OCIDefineObject( defnp, errhp, (OCIType *) xmltdo,
			       (dvoid **) &xml, &xmlsize,
			       (dvoid **) &xml_ind, (ub4 *) 0))
  {
    printf("FAILED: OCIDefineObject()\n");
    checkerr(errhp, status);
    return OCI_ERROR;
  }

  if(status = OCIStmtExecute( svchp, stmthp, errhp, (ub4) 1, (ub4) 0,
			      (CONST OCISnapshot*) 0, (OCISnapshot*) 0,
			      (ub4) OCI_DEFAULT))
  {
    printf("FAILED: OCIStmtExecute()\n");
    checkerr(errhp, status);
    return OCI_ERROR;
  }

  /* test some xml APIs */
  printf("\n\n=> Test some OCI XML and DOM APIs ......\n\n");

  if (test_xmlApi( xml, xpathexpr))
  {
    printf("FAILED: test_xmlApi()\n");
    return OCI_ERROR;
  }

  /* free instances */
  if (status = OCIObjectFree( envhp, errhp, (dvoid *) xml,
			      (ub2)OCI_OBJECTFREE_FORCE))
  {
    printf("FAILED: OCIObjectFree()\n");
    checkerr(errhp, status);
    return OCI_ERROR;
  }

  return OCI_SUCCESS;
}

/*--------------------------------------------------------*
/*               Test C++ XML DOM API
/*--------------------------------------------------------*/
sb4 ociapi::test_xmlApi( OCIXMLType* xml, text *xpathexpr)
{
  boolean result = 0;
  OCIXMLType *newxml = (OCIXMLType *) 0;
  OCIXMLType *xmltran = (OCIXMLType *) 0;
  sword  status = 0;

  /* For XML DOM APIs */
  OCIDuration dur = OCI_DURATION_SESSION;
  struct xmlctx *xctx = (xmlctx *)0;
  xmldocnode *doc = (xmldocnode *)0;
  ocixmldbparam params[1];


  /* Get an XML C and C++ context */
  params[0].name_ocixmldbparam = XCTXINIT_OCIDUR;
  params[0].value_ocixmldbparam = &dur;
  xctx = OCIXmlDbInitXmlCtx( envhp, svchp, errhp, params, 1);
  CXmlCtx* cxctxp = new CXmlCtx( xctx);

   /* convert the xmltype value to C++ DocumentRef */
  doc = (xmldocnode *) xml;
  DOMImplRef< CXmlCtx, xmlnode>
    impl_ref( cxctxp, new DOMImplementation< xmlnode>( FALSE));
  DocumentRef< xmlnode>* doc_refp = impl_ref.formDocument( (xmlnode*)doc);

  /* test xml unified C APIs */
  if (doXPath< CXmlCtx, xmlnode>( cxctxp, *doc_refp, (char*)xpathexpr)) {
    printf("FAILED: doXPath()\n");
    return OCI_ERROR;
  }

  /* free the statement handle */
  if (stmthp) {
    OCIHandleFree((dvoid *)stmthp, OCI_HTYPE_STMT);
  }

  /* free the xmlctx */
  OCIXmlDbFreeXmlCtx(xctx);

  /* free xml instances using OCIObjectFree */
  if (newxml &&
      (status = OCIObjectFree(envhp, errhp, (dvoid *) newxml,
			      (ub2)OCI_OBJECTFREE_FORCE))) {
   printf("FAILED: OCIObjectFree()\n");
   checkerr(errhp, status);
   return OCI_ERROR;
 }

 return OCI_SUCCESS;
}

/* Get member functions */

OCIEnv* ociapi::getEnv() const
{
  return envhp;
}

/* independent functions */

void checkerr( OCIError* errhp, sword status)
{
  text  msgbuf[512];
  sb4   errcode = 0;

  memset((void *) msgbuf, (int)'\0', (size_t)512);

  switch (status)
  {
  case OCI_SUCCESS: break;
  case OCI_SUCCESS_WITH_INFO:
    printf("status = OCI_SUCCESS_WITH_INFO\n");
    OCIErrorGet((dvoid *) errhp, (ub4) 1, (text *) NULL, &errcode,
                       msgbuf, (ub4) sizeof(msgbuf), (ub4) OCI_HTYPE_ERROR);
    printf("ERROR CODE = %d\n", errcode);
    printf("%.*s\n", 512, msgbuf);
    if (errcode == 436 || errcode == 437 || errcode == 438 || errcode ==
439)
      exit(1);
    break;
  case OCI_NEED_DATA:
    printf("status = OCI_NEED_DATA\n");
    break;
  case OCI_NO_DATA:
     printf("status = OCI_NO_DATA\n");
    break;
  case OCI_ERROR:
     printf("status = OCI_ERROR\n");
     OCIErrorGet((dvoid *) errhp, (ub4) 1, (text *) NULL, &errcode,
                       msgbuf, (ub4) sizeof(msgbuf), (ub4) OCI_HTYPE_ERROR);
     printf("ERROR CODE = %d\n", errcode);
     printf("%.*s\n", 512, msgbuf);
    if (errcode == 436 || errcode == 437 || errcode == 438 || errcode ==
439)
      exit(1);
    break;
  case OCI_INVALID_HANDLE:
     printf("status = OCI_INVALID_HANDLE\n");
    break;
  case OCI_STILL_EXECUTING:
     printf("status = OCI_STILL_EXECUTE\n");
    break;
  case OCI_CONTINUE:
     printf("status = OCI_CONTINUE\n");
    break;
  default:
    break;
  }

  return;
}

/* generic template to handle XPath processing */

template< typename TCtx, typename Tnode>
sb4 doXPath( TCtx* ctxp, DocumentRef< Tnode>& doc_ref, char* xpath_exp) {

  printf("Initializing Tools Factory\n");

  Factory< TCtx, Tnode>* fp = NULL;

  try 
  {
    fp = new Factory< TCtx, Tnode>( ctxp);
  }
  catch (FactoryException& fe)
  {
    unsigned ecode = fe.getCode();

    printf( "Failed to create factory, error %u\n", ecode);
    return ecode;
  }

  printf("Creating XPath processor\n");

  XPath::Processor< TCtx, Tnode>* prp = NULL;

  try 
  {
    prp = fp->createXPathProcessor( XPathPrCXml, NULL);
  }
  catch (FactoryException& fe1)
  {
    unsigned ecode = fe1.getCode();

    printf( "Failed to create XPath processor, error %u\n", ecode);
    return ecode;
  }

  printf( "Create DOM source for the DOm tree\n");

  InputSource* isrcp = new DOMSource< Tnode>( doc_ref);

  printf("Processing DOM tree with  '%s' ...\n", xpath_exp);

  XPathObject< Tnode>* objp = NULL;

  try
  {
    objp = prp->process (isrcp, (oratext*)xpath_exp);

  }
  catch (XPathException& xpe)
  {
    unsigned ecode = xpe.getCode();

    printf( "Failed to process the document, error %u\n", ecode);
    return ecode;
  }

  NodeSet< Tnode>* np = NULL;
  boolean varb = FALSE;
  double num = 0.0;
  oratext* str = NULL;
  unsigned i = 0;

  switch (objp->getObjType())
  {
  case XPOBJ_TYPE_NDSET:
    np = objp->getNodeSet();
    printf("NodeSet:\n");
    for (i = 0; i < np->getSize(); i++ )
    {
      NodeRef< Tnode>* nrefp = np->getNode( i);
      switch( nrefp->getNodeType())
      {
      case ELEMENT_NODE:
      case ATTRIBUTE_NODE:
	printf("Node Name : %s\n", 
	       nrefp->getNodeName());
	break;
      default:
	printf("Other Node \n");
	break;
      }
    }
    break;
  case XPOBJ_TYPE_BOOL:
    varb = objp->getObjBoolean();
    printf("Boolean Value : %d\n", varb);
    break;
  case XPOBJ_TYPE_NUM:
    num = objp->getObjNumber();
    printf("Numeric Value : %10.2f\n", num);
    break;
  case XPOBJ_TYPE_STR:
    str = objp->getObjString();
    printf("String Value : %s\n", str);
    break;
  default:
    printf( "Failed to create valid object\n");
  }
  return 0;
}
