/* Copyright (c) 2004, Mark Scardina, Ben Chang, Jinyu Wang  All rights reserved.  */
 
/*
   NAME
     cppextractGen.cpp - XPath Extraction Utility
 
   DESCRIPTION
     XPath Extraction Utility using C++ interfaces of the XPath processors
*/
#pragma warning( disable : 4290 ) 
#include <iostream.h>
#include "cppextract.hpp"


template< typename TCtx, typename Tnode> void printSubTree(NodeRef< Tnode>& nrefp);

template< typename TCtx, typename Tnode> unsigned processXPath(char* dname, char* xpath_exp) {

  TCtx* ctxp;

  cout << "XML C++ XPath Extract\n";

  try
  {
    ctxp = new TCtx();
  }
  catch (XmlException& e)
  {
    unsigned ecode = e.getCode();

    cout << "Failed to initialize XML context, error " << ecode << "\n";
    return ecode;
  }

  Factory< TCtx, Tnode>* fp;

  try 
  {
    fp = new Factory< TCtx, Tnode>( ctxp);
  }
  catch (FactoryException& fe)
  {
    unsigned ecode = fe.getCode();

    cout << "Failed to create factory, error " << ecode << "\n";
    return ecode;
  }

  XPath::Processor< TCtx, Tnode>* prp = NULL;

  try 
  {
    prp = fp->createXPathProcessor( XPathPrCXml, NULL);
  }
  catch (FactoryException& fe1)
  {
    unsigned ecode = fe1.getCode();

    cout << "Failed to create XPath processor, error " << ecode << "\n";
    return ecode;
  }

  
  InputSource* isrcp = new FileSource( (oratext*)dname);

  cout << "Processing " << dname;
  cout << " using " << "\"" << xpath_exp << "\"" << "\n";

  XPathObject< Tnode>* objp = NULL;

  try
  {
    objp = prp->process (isrcp, (oratext*)xpath_exp);

  }
  catch (XPathException& xpe)
  {
    unsigned ecode = xpe.getCode();

    cout << "Failed to process the document, error " << ecode << "\n";
    return ecode;
  }

  NodeSet< Tnode>* np;
  boolean varb;
  double num;
  oratext* str;
  unsigned i;
  
  switch (objp->getObjType())
  {
  case XPOBJ_TYPE_NDSET:
    np = objp->getNodeSet();
    cout << "NodeSet:\n";
    for (i = 0; i < np->getSize(); i++ )
    {
      NodeRef< Tnode>* nrefp = np->getNode( i);
      switch( nrefp->getNodeType())
      {
      case ELEMENT_NODE: 
        cout << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        printSubTree< Tnode>(*nrefp );
	    cout << "\n"; 
	break;
      case ATTRIBUTE_NODE:
	  cout << "Attribute Name : " << nrefp->getNodeName() << "\n";
	  break;
      default:
	  cout << "Node Value : " << nrefp->getNodeValue() << "\n";
	  break;
      }
    }
    break;

  case XPOBJ_TYPE_BOOL:
    varb = objp->getObjBoolean();
    cout << "Boolean Value : " << varb << "\n";
    break;
  case XPOBJ_TYPE_NUM:
    num = objp->getObjNumber();
    cout << "Numeric Value : " << num << "\n";
    break;
  case XPOBJ_TYPE_STR:
    str = objp->getObjString();
    cout << "String Value : " << str << "\n";
    objp = prp->process (isrcp, (oratext*)xpath_exp);
    break;
  default:
    cout << "Failed to create valid object\n";
  }
  return 0;
}
template< typename Tnode> void printSubTree( NodeRef< Tnode>& nref)
{
  oratext* tag = nref.getNodeName();
  if (tag == NULL)
  {
    cout << " Element has no name - error\n";
    return;
  }
  // print opening tag
   cout << "<" << tag;

  //Get attributes on element
  NamedNodeMap< Tnode>* attrs = nref.getAttributes(); 
  NamedNodeMapRef< Tnode> attref( nref, attrs);
  ub4 n_attrs = attref.getLength();
  NodeRef< Tnode>* attrefp = NULL;
  for (unsigned a = 0; a < n_attrs; a++)
  {
    Tnode* ap = attref.item( a);
    if (a == 0)
      attrefp = new NodeRef< Tnode>( nref, ap);
    else 
      attrefp->resetNode( ap);
// print attribute
    cout << " " << attrefp->getNodeName() << " = " << attrefp->getNodeValue();
  }
  cout << ">";
  NodeRef< Tnode>* nrefp = NULL;
  if (nref.hasChildNodes())
  {
    NodeList< Tnode>* lp = nref.getChildNodes();
    NodeListRef< Tnode> lref( nref, lp);

    ub4 len = lref.getLength();
    for (unsigned i = 0; i < len; i++)
    {
      Tnode* np = lref.item( i);
      if (i == 0) 
	    nrefp = new NodeRef< Tnode>( nref, np);
      else
	    nrefp->resetNode( np);

      if  (nrefp->getNodeType() == ELEMENT_NODE)
      {
//Continue iterration  
	printSubTree< Tnode>( *nrefp);
      }
      else if (nrefp->getNodeType() == TEXT_NODE)
//Print Text value
	cout << nrefp->getNodeValue(); 
    }	 
  }
//Print closing tag
  cout << "</" << tag << ">";
}

