-------------------------------------------------------------------------------
-- updateMRUDOM.sql
-- Description:
--   Update MRU using UPDATEXML function using DBMS_XMLDOM
-------------------------------------------------------------------------------
CONNECT &1/&2@&3

SET serveroutput ON SIZE 500000
SET LONG 10000
SET ECHO ON

-- Setup to know the SQL execution time
SET TIMING ON 
ALTER session SET SQL_TRACE=TRUE 
/

-- Select company inceptionDate information
SELECT extractValue(value(c),'/company/companyDetails/inceptionDate/text()') AS inceptionDate
FROM "company" c
WHERE extract(value(c),'/company/@atlasid').getStringVal()= 'atlasid0001'
/
DROP INDEX atlasid_idx
/
-- Create Functional Index on /company/@atlasid 
CREATE INDEX atlasid_idx ON "company" c(extractValue(value(c),'/company/@atlasid'))
/

-- Select company inceptionDate information
SELECT extractValue(value(c),'/company/companyDetails/inceptionDate/text()') AS inceptionDate
FROM "company" c
WHERE extract(value(c),'/company/@atlasid').getStringVal()= 'atlasid0001'
/
-- Print Line
CREATE OR REPLACE PROCEDURE printLine( linecon IN VARCHAR2 ) AS
  linemax VARCHAR2(250);
  line VARCHAR2(32767) := linecon;
BEGIN
  WHILE ( length(line) > 250 ) LOOP
      linemax := substr(line, 1, 250);
      dbms_output.put_line('| '||linemax);
      line:= substr(line, 251);
  END LOOP;
  dbms_output.put_line('| '||line);
END printline;
/
-- Print CLOB content
CREATE OR REPLACE PROCEDURE printClobOut(result IN OUT NOCOPY CLOB) AS
xmlstr VARCHAR2(32767);
line VARCHAR2(20000);
linemax VARCHAR(255);
BEGIN
  xmlstr := dbms_lob.SUBSTR(result,32767);
  LOOP
    EXIT WHEN xmlstr IS NULL;
    line := substr(xmlstr,1,instr(xmlstr,chr(10))-1);
    printLine(line);
    xmlstr := substr(xmlstr,instr(xmlstr,chr(10))+1);
    IF(instr(xmlstr,chr(10)) = 0) THEN 
      printLine(xmlstr);
     EXIT;
    END IF;
  END LOOP;
END printClobOut;
/
-- Update the companyDetails by adding/update its child element
-- such as the inceptionData Element(single with Text content)
CREATE OR REPLACE PROCEDURE UpdateElement(p_parent_xpath IN VARCHAR2, 
                                           p_where IN VARCHAR2,
                                           p_ename IN VARCHAR2,
                                           p_content IN VARCHAR2) AS
   v_temp XMLTYPE;
   v_clob CLOB;
   v_i NUMBER;
   v_buf VARCHAR2(2000);
   v_doc    dbms_xmldom.DOMDocument;
   v_ndoc   dbms_xmldom.DOMNode;
   v_nlist     dbms_xmldom.DOMNodeList;
   v_node dbms_xmldom.DOMNode;
   v_elem dbms_xmldom.DOMElement;
   v_nelem dbms_xmldom.DOMNode;
   v_text dbms_xmldom.DOMText;
   v_ntext dbms_xmldom.DOMNode;
BEGIN
   -- Select the companyDetails Element
   EXECUTE IMMEDIATE 
      'SELECT extract(value(c),'''||p_parent_xpath||''')'||
      'FROM "company" c where '||p_where
      INTO v_temp;

   -- Update the element by adding the inceptionData Element
   v_doc := dbms_xmldom.newDOMDocument (v_temp);
   v_ndoc := dbms_xmldom.makeNode(dbms_xmldom.getDocumentElement(v_doc));
 
   v_nlist:= dbms_xmldom.getElementsByTagName(v_doc,p_ename);
   IF dbms_xmldom.getLength(v_nlist) > 0 THEN
      dbms_output.put_line('Update the existing inceptionDate.');
      v_node := dbms_xmldom.item(v_nlist, 0);
      v_ntext := dbms_xmldom.getFirstChild(v_node);
      dbms_output.put_line(dbms_xmldom.getNodeValue(v_ntext));
      dbms_xmldom.setNodeValue(v_ntext, p_content);
    ELSE
      dbms_output.put_line('Create new inceptionDate.');
      v_elem := dbms_xmldom.createElement(v_doc,p_ename);
      v_nelem := dbms_xmldom.makeNode(v_elem);
      v_text := dbms_xmldom.createTextNode(v_doc, p_content);
      v_ntext := dbms_xmldom.makeNode(v_text);
      v_ntext := dbms_xmldom.appendChild(v_nelem, v_ntext);
      v_nelem := dbms_xmldom.appendChild(v_ndoc, v_nelem);
    END IF;
    -- Bug2506819, getxmltype will always return null
    -- v_temp:=dbms_xmldom.getxmltype;   
      
    ---------------------------------------------------------------------------
    -- Currently we have two options:
    --  1. Print out the DOM in CLOB and buildup XMLType
    --  2. Keep the XMLType variable around to get the result in XMLTYPE
    ---------------------------------------------------------------------------
    --dbms_lob.createtemporary(v_clob,TRUE);
    --dbms_xmldom.writetoClob(dbms_xmldom.makenode(v_doc), v_clob);
    -- printClobOut(v_clob);
       
    -- v_clob := v_temp.getClobVal();
    -- printClobOut(v_clob);

    -- Update the company table 
    --UPDATE "company" p
    --  SET value(p) = UPDATEXML(value(p),p_parent_xpath,XMLTYPE(v_clob));
    EXECUTE IMMEDIATE 
    'UPDATE "company" c
      SET value(c) = UPDATEXML(value(c),:bv1,:bv2) WHERE ' || p_where
    using p_parent_xpath,v_temp;

    -- Free resources
    dbms_xmldom.freedocument(v_doc);
    --dbms_lob.freetemporary(v_clob);

EXCEPTION
   WHEN dbms_xmldom.INDEX_SIZE_ERR THEN 
      raise_application_error(-20120, 'Index Size error');
   WHEN dbms_xmldom.DOMSTRING_SIZE_ERR THEN
      raise_application_error(-20120, 'String Size error');
   WHEN dbms_xmldom.HIERARCHY_REQUEST_ERR THEN
      raise_application_error(-20120, 'Hierarchy request error');
   WHEN dbms_xmldom.WRONG_DOCUMENT_ERR THEN
      raise_application_error(-20120, 'Wrong doc error');
   WHEN dbms_xmldom.INVALID_CHARACTER_ERR THEN
      raise_application_error(-20120, 'Invalid Char error');
   WHEN dbms_xmldom.NO_DATA_ALLOWED_ERR THEN
      raise_application_error(-20120, 'Nod data allowed error');
   WHEN dbms_xmldom.NO_MODIFICATION_ALLOWED_ERR THEN
      raise_application_error(-20120, 'No mod allowed error');
   WHEN dbms_xmldom.NOT_FOUND_ERR THEN
      raise_application_error(-20120, 'Not found error');
   WHEN dbms_xmldom.NOT_SUPPORTED_ERR THEN
      raise_application_error(-20120, 'Not supported error');
   WHEN dbms_xmldom.INUSE_ATTRIBUTE_ERR THEN
      raise_application_error(-20120, 'In use attr error');
END;
/
show errors;

SELECT extract(value(c),'/company/*[name()="companyDetails"]')
FROM "company" c 
WHERE extractValue(value(c),'/company/@atlasid')= 'atlasid0001'
/      
DECLARE
   v_begDate DATE;
   v_endDate DATE;
   v_dur NUMBER;
BEGIN
   v_begDate := SYSDATE;
   UpdateElement('/company/*[name()="companyDetails"]',
                 'extractValue(value(c),''/company/@atlasid'')= ''atlasid0001''',
                 'inceptionDate',
                 'inceptionData (Example)(updated)');
   v_endDate := SYSDATE;
   

   -- Report result
   v_dur := to_number(to_char(v_endDate,'SSSSS'))- to_number(to_char(v_begDate,'SSSSS'));
   dbms_output.put_line('Elapsed(PL/SQL):'|| v_dur ||' seconds');
   END;
/

-- Select company inceptionDate information
SELECT extractValue(value(c),'/company/companyDetails/inceptionDate/text()') AS inceptionDate
FROM "company" c
WHERE extractValue(value(c),'/company/@atlasid')= 'atlasid0001'
/
-- Quit


