import java.sql.*;
import java.util.*;
import java.io.*;
import oracle.jdbc.driver.*;
import oracle.xml.sql.query.*;
import oracle.xml.dbaccess.*;
import oracle.xml.parser.v2.*;

public class DBAccessDemo
{

  public static void main (String[] args) 
  {
    if (args.length < 5)
       System.err.println("Usage: \n java DBAccessDemo <hostname> <port> <sid> <userid> <passwd>");
    DBAccessDemo demo =  new DBAccessDemo(args[0], args[1], 
                                                    args[2], args[3], args[4]);
    demo.createXMLCLOBTable("testxmltype");
    demo.listXMLCLOBTables();
    demo.replaceXMLCLOBData("testxmltype", "mybooklist", "booklist.xml");
    demo.getXMLCLOBData("testxmltype", "mybooklist");
    demo.finishDemo();
  }

  DBAccessdemo(String h, String p, String i, String u, String pw)
  {
    hostname = h;
    port = p;
    instancename = i;
    username = u;
    password = pw;  
    
    try
    {
       log = new PrintWriter (new FileWriter(logname));
       makeConnection();
    }catch(Exception e)
    {
       e.printStackTrace(System.out);
    }
    db = new DBAccess();
  }

  public void finishDemo()
  {
     try
     {
        closeConnection();
        log.flush();
        log.close();
     }catch (Exception e)
     {
        e.printStackTrace(System.out);
     }
  }   

  void createXMLCLOBTable(String tableName)
  {
      log.println("\nDemo for createXMLTables():");
      try {
         db.createXMLCLOBTable(con, tableName);
      } catch (Exception ex) {
          log.println("Error creating XML CLOB table: " + ex.getMessage()); 
      }
      log.println("Table +'" + tableName + "' successfully created.");
      return;
  }

  void replaceXMLCLOBData( String tablename, String xmlname, String filename)
  {
      log.println("\nDemo for replaceXMLCLOBData() (similar to insert):");
      String xmldata = loadFile(filename);
      try 
      {
         db.replaceXMLCLOBData(con, tablename, xmlname, xmldata);
      } catch (Exception ex) {
          log.println("Error inserting into XML CLOB table: " + ex.getMessage());
      }
      log.println("XML Data from +'" + filename + "' successfully replaced in table '" 
                  + tablename + "'.");
      return;
  }

  void getXMLCLOBData(String tablename, String xmlname)
  {
      log.println("\nDemo for getXMLCLOBData():");
      String xmlText=null;
      try 
      {
         xmlText=db.getXMLCLOBData(con, tablename, xmlname);
      } catch (Exception ex) 
      {
         log.println("Error getting XML data: " + ex.getMessage());
        return;
      }
      log.println("XML CLOB data fetched: ");
      log.println(xmlText);
      return;
  }

  
  void listXMLCLOBTables() 
  {
      // list all recognized XML CLOB tables
      log.println("\nDemo for listXMLCLOBTables():");
      String tableNames[];
      int i;

      try 
      {
         tableNames=db.getXMLCLOBTableNames(con,"");
         
         for (i=0;i<tableNames.length;i++) 
         {
            log.println("tablenamename="+tableNames[i]);
         }
      } catch (Exception ex) 
      {
         log.println("Error listing XML CLOB tables: " + ex.getMessage());   
      }
  }
     
  void closeConnection() throws SQLException 
  {
    if  (con != null) {
        con.close();
        con=null;
    }
  }

  void makeConnection() throws SQLException 
  {

    String thinConn= "jdbc:oracle:thin:@"+
                               hostname+":"+
                               port+":"+
                               instancename;
    String defaultConn = "jdbc:oracle:kprb:";
    Driver drv;
    if  (con != null) {
        con.close();
        con=null;
    }
    // Load the Oracle JDBC driverm (alternative method)
    //DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());
    try {
      drv = (Driver)Class.forName(driverclass).newInstance();
    }
    catch (Exception e) {
      throw new SQLException("Error Loading JDBC Driver "+driverclass);
    }
    if (insideOracle()) 
    {
         con=DriverManager.getConnection(defaultConn,username,password);
    } else 
    {
         con=DriverManager.getConnection(thinConn,username,password);
    }
  }


  boolean insideOracle() 
  {
    // If oracle.server.version is non null, we're running in the database
    String ver = System.getProperty("oracle.server.version");
    return (ver != null && !ver.equals(""));
  }

  String loadFile (String fileName) {
     StringBuffer textBuffer=new StringBuffer(1000);
     String line="";
     textBuffer.append("");
     try {
        BufferedReader reader=new BufferedReader(new FileReader(fileName));
        while (true) {
           line=reader.readLine();
           if (line==null) break;
           textBuffer.append(line);
        }
        return textBuffer.toString();
     } catch (Exception ex1) {
        return "";
     }
  }


  String username;
  String password;
  String hostname;
  String port;
  String instancename;
  XMLDocument xmlDoc=null;
  String xmlFileName=null;
  String SQLText="";
  String dataSourceString="";

  String driverclass   = "oracle.jdbc.driver.OracleDriver";
  Connection con=null;

  String logname = "DBaccess.log";
  PrintWriter log = null;
  DBAccess db = null;
}
