package oracle.xml.sample;
import java.net.URL;
import oracle.xml.async.DOMBuilder;
import oracle.xml.async.DOMBuilderErrorEvent;
import oracle.xml.async.DOMBuilderErrorListener;
import oracle.xml.async.DOMBuilderEvent;
import oracle.xml.async.DOMBuilderListener;
import oracle.xml.parser.v2.XMLDocument;

public class MultipleParse extends Object 
      implements DOMBuilderListener, DOMBuilderErrorListener {
  int numArgs,i;
  String Args[];
  DOMBuilder tParser;

  public MultipleParse(String[] args) {
    Args=args;
  }
  
  public void parse() {
    for(i=0;i<Args.length;i++) {
      // new instance of the asynchronous parser
      tParser=new DOMBuilder(i);
      // add this Listener object to be notified when parsing is complete
      tParser.addDOMBuilderListener(this);
      // or when an error occurs
      tParser.addDOMBuilderErrorListener(this);
      System.out.println("Start parsing "+Args[i]);
      try {
        tParser.parse(new URL("file:"+(String)Args[i]));
      } catch(Exception e) {
        System.out.println(e.toString());
      }
    }
    System.out.println("Multiple files parsed in background threads");
  }
  
  public static void main(String[] args) {
    MultipleParse mParse = new MultipleParse(args);
    mParse.parse();
  }
  // Implementing DOMBuilderListener Interface
  // Method called by DOMBuilder when document parsing starts
  public void domBuilderStarted(DOMBuilderEvent p0) {
    System.out.println("DOMBuilderStarted");
  }
  // Method called by DOMBuilder when document parsing returns an error.
  public void domBuilderError(DOMBuilderEvent p0) {
    System.out.println("DOMBuilderError");
  }
  // Method called by DOMBuilder when document parsing is completed.
  public void domBuilderOver(DOMBuilderEvent p0) {
    DOMBuilder parser;
    XMLDocument xmlDoc;
    int id;
    // Get a reference to the parser instance that finished parsing.
    parser=(DOMBuilder)p0.getSource();
    // Get the parser id to identify the file being parsed
    id=parser.getId();
    System.out.println("Parse completed for file "+Args[id]);
    // get the dom tree
    xmlDoc=parser.getDocument();
    // You can add custom code here to work with the parsed document.
    try 
    {
      xmlDoc.print(System.out); 
    } catch (Exception ex) 
    {
      ex.printStackTrace();
    } finally 
    {
    }
    
  }

  // This method is called when parsing error occurs.
  public void domBuilderErrorCalled(DOMBuilderErrorEvent p0) {
    int id=((DOMBuilder)p0.getSource()).getId();
    System.out.println("Parse error for "+Args[id]+": "+
                       p0.getException().getMessage());
  }
}
