-------------------------------------------------------------------------------
-- FileName: testDBMS_XMLSTORE.sql
-- Description: 
--  test the DBMS_XMLSTORE package
-- Modified (MM/DD/YYYY)
--   jiwang   05/14/2003 - Update the setDateFormat and limit the # rows 
--                         to process.
--   mscardin 05/13/2003 - Fixed setDateFormat.
--   jiwang  12/02/2003 - Clean up
--   jiwang  11/27/2003 - Create
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Example 1: Scalablity Comparison
-------------------------------------------------------------------------------
DROP TABLE test;
DROP TABLE result;
DROP TABLE temp_clob;

SELECT count(1) FROM sales;

CREATE TABLE test AS SELECT * FROM sales where rownum<1000;
--WHERE 0=1;
CREATE TABLE result AS SELECT * FROM sales WHERE 0=1;
CREATE TABLE temp_clob( doc CLOB);

-- Check the current status of the data
SELECT count(1) FROM test;

-- Using DBMS_XMLGen
SET serveroutput ON  SIZE 100000
SET echo ON
DECLARE
   qryCtx DBMS_XMLGen.ctxHandle;
   v_clob CLOB;
 BEGIN
   -- Query out the content
   qryCtx := DBMS_XMLGen.newContext('select * from test');
   v_clob := DBMS_XMLGen.getXml(qryCtx);
   DBMS_OUTPUT.put_line('CLOB size = '||DBMS_LOB.getlength(v_clob));
   INSERT INTO temp_clob VALUES(v_clob);
   DBMS_XMLGEN.closeContext(qryCTx);
END;
/
COMMIT;

-- Check the current status of the data
SELECT count(1) FROM temp_clob;
SELECT dbms_lob.getLength(doc) FROM temp_clob;


-- Using DBMS_XMLStore
SET timing ON 
DECLARE
   v_clob CLOB;
   savCtx DBMS_XMLSTORE.ctxType;
   v_rows NUMBER;
BEGIN
   -- Query out the content
   SELECT doc INTO v_clob FROM temp_clob;
   -- Save the content
   savCtx := DBMS_XMLSTORE.newContext('RESULT');
   savCtx := DBMS_XMLSTORE.newContext('RESULT');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'PROD_ID');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'CUST_ID');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'TIME_ID');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'CHANNEL_ID');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'PROMO_ID');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'QUANTITY_SOLD');
   DBMS_XMLSTORE.SETUPDATECOLUMN(savCtx, 'AMOUNT_SOLD');

   v_rows := DBMS_XMLSTORE.insertxml(savCtx,v_clob);
   DBMS_XMLSTORE.closeContext(savCtx);
   dbms_output.put_line(v_rows || ' rows inserted...');
END;
/
show errors;

-- Check the current status of the data
SELECT count(1) FROM result;

-------------------------------------------------------------------------------
-- Example 2: Simple Test
-------------------------------------------------------------------------------
SET timing ON 
-- Using DBMS_XMLSave
DECLARE
   v_clob CLOB;
   savCtx DBMS_XMLSAVE.ctxType;
   v_rows NUMBER;
BEGIN
   -- Query out the content
   SELECT doc INTO v_clob FROM temp_clob;
    -- Save the content
   savCtx := DBMS_XMLSAVE.newContext('RESULT');
   DBMS_XMLSAVE.SETDATEFORMAT(savCtx, 'dd-MMM-yy');
   v_rows := DBMS_XMLSAVE.insertxml(savCtx,v_clob);
    DBMS_XMLSAVE.closeContext(savCtx);
   dbms_output.put_line(v_rows || ' rows inserted...');
END;
/
show errors;

-- End

