set echo off
-------------------------------------------------------------------------------
-- FileName: testResourceManagement.sql
-- Description: 
--  Create resources in the XML DB repository
-- Modified (MM/DD/YYYY)
--   jiwang  12/02/2003 - Clean up
--   jiwang  11/27/2003 - Create
-------------------------------------------------------------------------------
set echo on
-------------------------------------------------------------------------------
-- Delete resources if needed
-------------------------------------------------------------------------------
EXEC DBMS_XDB.DeleteResource ('/public/customer1.xml');
EXEC DBMS_XDB.DeleteResource ('/public/customer2.xml');

COMMIT;

-------------------------------------------------------------------------------
-- Create new resources from the customer table
-------------------------------------------------------------------------------
SET serveroutput ON 
DECLARE
 res BOOLEAN; 
 xr  REF XMLType; 
BEGIN
  FOR po_rec IN (SELECT ROWNUM id, REF(p) xref FROM customer p
                 ORDER BY rowid) 
  LOOP
    res:=DBMS_XDB.createResource('/public/customer'||po_rec.id||'.xml',
                                     po_rec.xref); 
    IF res = TRUE THEN 
      DBMS_OUTPUT.PUT_LINE('One resrouce'||
              '/public/customer'||po_rec.id||'.xml'||' created.');
    END IF;
  END LOOP; 
END; 
/
-- List all the resource URLs in /public dir
SELECT any_path 
FROM RESOURCE_VIEW
WHERE UNDER_PATH(res,'/public',1) >0
order by 1;

-- Show the content of customer1.xml
SELECT res 
FROM RESOURCE_VIEW
WHERE any_path='/public/customer1.xml';

-------------------------------------------------------------------------------
-- Create new directory, the README.txt
-------------------------------------------------------------------------------
DECLARE
   retb BOOLEAN; 
BEGIN
   retb := DBMS_XDB.createFolder('/public/important_customer'); 
   COMMIT; 
END; 
/

DECLARE
  res BOOLEAN; 
BEGIN
  res := DBMS_XDB.createResource('/public/important_customer/README.txt',
   'This folder list all of the US customer who are important to our business'); 
  COMMIT; 
END; 
/
-------------------------------------------------------------------------------
-- Create a link
-------------------------------------------------------------------------------
EXEC DBMS_XDB.link('/public/customer1.xml','/public/important_customer','SteveJones.xml');

-- Check the reference count
select extractvalue(res, '/Resource/RefCount'), any_path 
from resource_view
WHERE equals_path(res,'/public/customer1.xml')>0;

-- List all the resource URLs in /public dir
SELECT any_path 
FROM resource_view
WHERE equals_path(res,'/public/important_customer/SteveJones.xml')>0;

SELECT any_path
FROM RESOURCE_VIEW
WHERE UNDER_PATH(res,'/public/important_customer/',1) >0
order by 1;

EXEC DBMS_XDB.DeleteResource ('/public/customer1.xml');

-- End


