-------------------------------------------------------------------------------
-- FileName: checkXMLValidation.sql
-- Description: 
--   Insert a customer record with the phone number in wrong pattern
-- Modified (MM/DD/YYYY)
--   jiwang  12/02/2003 - Clean up
--   jiwang  04/15/2003 - Create
-------------------------------------------------------------------------------
set echo on
DROP TRIGGER customer_insert;

-- Though the record has error, it can be inserted
INSERT INTO customer VALUES(
  XMLType('<Customer xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:noNamespaceSchemaLocation="http://localhost:8080/public/contact_simple_ann.xsd">
  <NAME>Steve Joes</NAME>
  <EMAIL>Steve.Joes@example.com</EMAIL>
  <ADDRESS>Someroad, Somecity, Redwood Shores, CA 94065, U.S.A</ADDRESS>
  <PHONE>6505723456</PHONE>
  <DESCRIPTION>Very Important US Customer</DESCRIPTION>
</Customer>')); 

ROLLBACK;

-- Create Trigger
CREATE OR REPLACE TRIGGER customer_insert
AFTER INSERT ON customer
FOR EACH ROW 
DECLARE
  doc XMLType;
BEGIN
  doc := :new.sys_nc_rowinfo$;
  XMLType.schemavalidate(doc);
END;
/
show errors;

-- This time you will get an error
INSERT INTO customer VALUES(
  XMLType('<Customer xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:noNamespaceSchemaLocation="http://localhost:8080/public/contact_simple_ann.xsd">
  <NAME>Steve Joes</NAME>
  <EMAIL>Steve.Joes@example.com</EMAIL>
  <ADDRESS>Someroad, Somecity, Redwood Shores, CA 94065, U.S.A</ADDRESS>
  <PHONE>6505723456</PHONE>
  <DESCRIPTION>Very Important US Customer</DESCRIPTION>
</Customer>'));

-- Update the record
INSERT INTO customer VALUES(
  XMLType('<Customer xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xsi:noNamespaceSchemaLocation="http://localhost:8080/public/contact_simple_ann.xsd">
 <NAME>Steve Joes</NAME>
 <EMAIL>Steve.Joes@example.com</EMAIL>
 <ADDRESS>Someroad, Somecity, Redwood Shores, CA 94065, U.S.A
 </ADDRESS>
 <PHONE>(650)572-3456</PHONE>
 <DESCRIPTION>Very Important US Customer</DESCRIPTION>
</Customer>')); 

-- Verify the rows 
SELECT x.isSchemaValid() 
FROM customer x;

DROP TRIGGER customer_insert;

-- End
